from datetime import datetime
from pathlib import Path

from django_scaffolding_tools.django.builders import build_model_serializer_template_data
from django_scaffolding_tools.django.handlers import IntegerFieldHandler, CharFieldHandler, ForeignKeyFieldHandler, \
    DateFieldHandler, DateTimeFieldHandler, DecimalFieldHandler, BooleanFieldHandler, DateTimeCharFieldHandler, \
    EmailFieldHandler, JSONFieldHandler
from django_scaffolding_tools.django.parsers import parse_for_django_classes
from django_scaffolding_tools.parsers import parse_file_for_ast_classes
from django_scaffolding_tools.writers import ReportWriter
from django_scaffolding_tools.utils.core import quick_write


def write_model_serializer_from_models_file(models_file: Path, output_file: Path, write_intermediate: bool = False,
                                            camel_case=False):
    """Parses a Django models.py file and generates the serializer for all the models."""
    # 1 Convert model.py to an ast json file.
    ast_dict = parse_file_for_ast_classes(models_file)
    if write_intermediate:
        model_filename = 'models.py'
        quick_write(ast_dict, f'ast_{model_filename}.json', output_subfolder='django')
    # 2 Parse AST json dictionary for Django Model data
    model_data = parse_for_django_classes(ast_dict)
    if write_intermediate:
        model_filename = 'models.py'
        quick_write(model_data, f'model_data_{model_filename}.json', output_subfolder='django')
    # 3 Build serializer data form Django model data
    serializer_data = build_model_serializer_template_data(model_data, add_source_camel_case=camel_case)
    if write_intermediate:
        model_filename = 'models.py'
        quick_write(serializer_data, f'serializer_data_{model_filename}.json', output_subfolder='django')
    writer = ReportWriter()
    writer.write('drf_model_serializers.py.j2', output_file, template_data=serializer_data)


def write_model_factories_from_models_file(models_file: Path, output_file: Path, write_intermediate: bool = False,
                                           camel_case=False):
    """Parses a Django models.py file and generates the serializer for all the models."""
    # 1 Convert model.py to an ast json file.
    ast_dict = parse_file_for_ast_classes(models_file)
    if write_intermediate:
        model_filename = 'models.py'
        quick_write(ast_dict, f'ast_{model_filename}.json', output_subfolder='django')
    # 2 Parse AST json dictionary for Django Model data
    model_data = parse_for_django_classes(ast_dict)
    if write_intermediate:
        model_filename = 'models.py'
        quick_write(model_data, f'model_data_{model_filename}.json', output_subfolder='django')
    handlers = [
        IntegerFieldHandler(),
        CharFieldHandler(),
        DateTimeCharFieldHandler(),
        ForeignKeyFieldHandler(),
        DateFieldHandler(),
        DateTimeFieldHandler(),
        DecimalFieldHandler(),
        BooleanFieldHandler(),
        EmailFieldHandler(),
        JSONFieldHandler(),
    ]

    for i in range(len(handlers)):
        if i < len(handlers) - 1:
            handlers[i].set_next(handlers[i + 1])

    main_handler = handlers[0]
    lines = list()
    imports = f"""# Generated with django_scaffolding_tool {datetime.now()}
import string
from datetime import datetime

from django.utils import timezone
from factory import Iterator, SelfAttribute, Trait
from factory import LazyAttribute
from factory import SubFactory
from factory.django import DjangoModelFactory
from factory.fuzzy import FuzzyText
from faker import Factory as FakerFactory

faker = FakerFactory.create()\n"""
    lines.append(imports)
    for fp_data in model_data['classes']:
        lines.append(f'class {fp_data["name"]}Factory(DjangoModelFactory):\n')
        lines.append(f'\tclass Meta:\n')
        lines.append(f'\t\tmodel = {fp_data["name"]}\n')
        lines.append("\n")
        for att in fp_data['attributes']:
            result = main_handler.handle(att)
            if result is None:
                lines.append(f'\t# {att["name"]} {att["data_type"]} NOT supported\n')
            else:
                lines.append(f'\t{result["name"]} = {result["factory_field"]}\n')
        lines.append('\n')
        # lines.append("#" * 80)
    with open(output_file, 'w') as py_file:
        py_file.writelines(lines)



from typing import Any, List, Dict

from django_scaffolding_tools.enums import NativeDataType
from django_scaffolding_tools.parsers import SERIALIZER_FIELDS


def build_serializer_template_data(model_list: List[Dict[str, Any]]) -> Dict[str, Any]:
    """Builds a dictionary to be used in the Jinja template"""
    template_data = dict()
    template_data['imports'] = ['from rest_framework import serializers']
    template_models = list()
    template_data['classes'] = template_models
    for model in model_list:
        serializer_data = dict()
        serializer_data['name'] = f'serializers.{model["name"]}Serializer(serializers.Serializer)'
        serializer_data['attributes'] = list()
        for attribute in model['attributes']:
            serializer_attribute = dict()
            serializer_attribute['name'] = attribute['name']
            keywords = list()
            for keyword in attribute['serializer'].get('keywords'):
                if isinstance(keyword['value'], str):
                    keywords.append(f'{keyword["name"]}=\'{keyword["value"]}\'')
                else:
                    keywords.append(f'{keyword["name"]}={keyword["value"]}')
            field_vars = ', '.join(keywords)
            serializer_attribute['field'] = f'serializers.{attribute["serializer"]["field_type"]}({field_vars})'
            serializer_data['attributes'].append(serializer_attribute)
        template_models.append(serializer_data)
    return template_data


def build_serializer_info(field_type: str, keywords: List[Dict[str, Any]]):
    field_data = {'field_type': field_type, 'keywords': keywords}
    return field_data


def build_serializer_data(model_list: List[Dict[str, Any]],
                          serializer_fields: Dict[str, Any] = SERIALIZER_FIELDS) -> List[Dict[str, Any]]:
    for model in model_list:
        for attribute in model['attributes']:
            data_type = attribute['data_type']
            keywords = list()
            if data_type == NativeDataType.STRING:
                pattern_type = attribute.get('pattern_type')
                keywords.append({'name': 'max_length', 'value': attribute['length']})
                if pattern_type is None:
                    serializer_field = serializer_fields.get(data_type)
                    if serializer_field is not None:
                        if attribute.get('alias'):
                            keywords.append({'name': 'source', 'value': attribute['alias']})
                    attribute['serializer'] = {'field_type': serializer_field['field'], 'keywords': keywords}
                else:
                    serializer_field = serializer_fields.get(pattern_type)
                    if attribute.get('alias'):
                        keywords.append({'name': 'source', 'value': attribute['alias']})
                    attribute['serializer'] = {'field_type': serializer_field['field'], 'keywords': keywords}
            else:
                serializer_field = serializer_fields.get(data_type)
                if attribute.get('alias'):
                    keywords.append({'name': 'source', 'value': attribute['alias']})
                if serializer_field is None:
                    attribute['serializer'] = {'field_type': f'{attribute["data_type"]}Serializer',
                                               'keywords': keywords}
                else:
                    attribute['serializer'] = {'field_type': serializer_field['field'], 'keywords': keywords}

    return model_list


def build_django_factory(model_name: str, model_data: Dict[str, Any]):
    data_type = ''
    name = ''
    if 'timestamp' in name and data_type == 'IntegerField':
        # LazyAttribute(lambda x: faker.date_time_between(start_date="-1y", end_date="now",
        #                                                                     tzinfo=timezone(settings.TIME_ZONE)).timestamp())
        pass
    elif 'date'in name and data_type == 'IntegerField':
        pass




from autoslug import AutoSlugField
from django.conf import settings

from django.db.models import JSONField
from django.db import models, IntegrityError
from django.utils.translation import ugettext_lazy as _
from model_utils.models import TimeStampedModel

from .exceptions import ClinicException
from ..core.models import Human, Auditable
# Create your models here.
from ..patients.models import Patient


def build_slug(physician):
    return '{} {}'.format(physician.last_name, physician.first_name)


def upload_logo(instance, filename):
    extension = filename.split('.')[-1]
    return f'{instance.folder}/{instance.slug}-logo.{extension}'


def get_default_discount_config():
    json_data = settings.EMR_PRACTICE['DEFAULT_DISCOUNT_CONFIG']
    return json_data


class Clinic(models.Model):
    name = models.CharField(_('Name'), max_length=100)
    short_name = models.CharField(_('Short name'), max_length=20, null=True, blank=True)
    slug = AutoSlugField(populate_from='name', unique=True, always_update=True)
    folder = AutoSlugField(populate_from='name', unique=True)
    owner = models.ForeignKey(settings.AUTH_USER_MODEL, on_delete=models.PROTECT, verbose_name=_('Owner'),
                              related_name='owned_clinics')
    logo = models.FileField(upload_to=upload_logo, verbose_name=_('Clinic logo'), null=True, blank=True)
    default_hospital = models.ForeignKey('CareTakingFacility', verbose_name=_('Default Hospital'), null=True,
                                         blank=True, on_delete=models.SET_NULL)
    discount_config = JSONField(_('Discount configuration'), null=True, blank=True, default=get_default_discount_config)
    settings = JSONField(_('Settings'), null=True, blank=True, default=get_default_discount_config)

    def __str__(self):
        return self.name

    class Meta:
        ordering = ('name',)

    def add_member(self, clinic_member):
        try:
            return ClinicMembership.objects.create(clinic_member=clinic_member, clinic=self)
        except IntegrityError:
            msg = _('{} is already a member of {}'.format(clinic_member, self))
            raise ClinicException(msg)

    def balance(self):
        from ..finance.models import Invoice
        return Invoice.objects.outstanding_balance(self)


class Company(models.Model):
    name = models.CharField(_('Name'), max_length=100)
    short_name = models.CharField(_('Short name'), max_length=20, null=True, blank=True)
    national_id = models.CharField(_('National id'), max_length=20, null=True, blank=True)
    verification_digit = models.CharField(_('Verification digit'), max_length=4, null=True, blank=True)
    clinic = models.ForeignKey(Clinic, verbose_name=_('Clinic'), related_name='companies', on_delete=models.PROTECT)
    order = models.IntegerField(_('Order'), default=1)
    settings = JSONField(_('Settings'), null=True, blank=True, default=get_default_discount_config)
    slug = AutoSlugField(populate_from='name', unique=True, always_update=True)
    folder = AutoSlugField(populate_from='name', unique=True)
    logo = models.ImageField(upload_to=upload_logo, verbose_name=_('Company logo'), null=True, blank=True)

    class Meta:
        ordering = ('order', 'name')

    def __str__(self):
        return self.name



from typing import List, Dict, Any

from tqdm import tqdm

from django_scaffolding_tools._experimental.docker.commnads import run_commands, run_command_with_grep


def get_containers(regexp: str):
    commands = ['docker', 'ps', '-a']
    results = run_command_with_grep(commands, regexp)
    container_list = list()
    if len(results) > 0:
        for r in results:
            data = r.split(' ')
            if len(data[0]) != 0:
                container_list.append({'container_id': data[0], 'image': data[3], 'name': data[-1]})
    return container_list


def delete_containers(containers: List[Dict[str, Any]], reg_expression: str):
    if len(containers) != 0:
        for i, container in enumerate(containers):
            print(f'{i} {container["container_id"]} {container["image"]} {container["name"]}')
        container_to_delete = input(f'Type the number of the container to delete (#, None [n], All [all]):')
        if container_to_delete.isdigit():
            container_id = int(container_to_delete)

            delete_container_command = ['docker', 'rm', containers[container_id]['container_id']]
            dc_res, errors = run_commands(delete_container_command)
            print(f'Deleted container {dc_res}')
        elif container_to_delete.lower() == 'all':
            for container in tqdm(containers):
                delete_container_command = ['docker', 'rm', container['container_id']]
                dc_res, errors = run_commands(delete_container_command)
                print(f'Deleted container {dc_res}')
        else:
            print('No containers were deleted')
    else:
        print(f'No container found for {reg_expression}')


def main(reg_expression: str):
    containers = get_containers(reg_expression)
    delete_containers(containers, reg_expression)


if __name__ == '__main__':
    r = r"cupos"
    main(r)



from typing import List, Optional

from django_scaffolding_tools._experimental.docker.commnads import run_commands, run_command_with_grep
from django_scaffolding_tools._experimental.docker.containers import get_containers, delete_containers
from django_scaffolding_tools._experimental.docker.enums import ProjectPostgresRegExp, TerminalColor


def get_volumes(regexp):
    commands = ['docker', 'volume', 'ls']
    results = run_command_with_grep(commands, regexp)
    volume_list = list()
    for r in results:
        data = r.split(' ')
        if len(data[0]) != 0:
            volume_list.append({'driver': data[0], 'name': data[-1]})
    return volume_list


def split_and_clean(line: str) -> List[str]:
    line_parts = line.split(' ')
    final = [x for x in line_parts if len(x) > 0]
    return final


def get_images(regexp):
    commands = ['docker', 'image', 'ls']
    results = run_command_with_grep(commands, regexp)
    image_list = list()
    for r in results:
        data = split_and_clean(r)
        if len(data) > 0:
            image_list.append({'name': data[0], 'image_id': data[2]})
    return image_list


def do_cleanup(regexpression: str):
    containers = get_containers(regexpression)
    delete_containers(containers, regexpression)

    d_vols = get_volumes(regexpression)
    # print(d_vols)
    if len(d_vols) != 0:
        for i, d_vol in enumerate(d_vols):
            volume_name = d_vol["name"]
            print(f'({i}) {volume_name}')

        volume_to_delete = input('Volume to delete (#, None [n], All [a]):')
        delete_volume_command = ['docker', 'volume', 'rm']
        if volume_to_delete.lower() == 'a':
            for d_vol in d_vols:
                delete_volume_command.append(d_vol['name'])
        elif volume_to_delete.isdigit():
            delete_volume_command.append(d_vols[int(volume_to_delete)]['name'])
        else:
            print('Not deleting any volumes')
        if len(delete_volume_command) > 3:
            v_res, v_errors = run_commands(delete_volume_command)
            print(f'Deleted {v_res}')
    else:
        print(f'No volumes found for {regexpression}')

    ####################################################################
    # IMAGES

    d_images = get_images(regexpression)
    if len(d_images) > 0:
        for i, d_image in enumerate(d_images):
            print(f'({i}) {d_image["name"]}')
        image_to_delete = input('Image to delete (#, n, a): ')
        delete_image_command = ['docker', 'image', 'rm']
        if image_to_delete.lower() == 'a':
            for d_image in d_images:
                delete_image_command.append(d_image['name'])
        elif image_to_delete.isdigit():
            delete_image_command.append(d_images[int(image_to_delete)]['name'])
        else:
            print('Not deleting any images')
        if len(delete_image_command) > 3:
            i_res, i_errors = run_commands(delete_image_command)
            print(f'Deleted {i_res}')
    else:
        print(f'No images found for {regexpression}')


def bold_text(text: str, color: Optional[TerminalColor] = None):
    if color is None:
        color_code = ''
    else:
        color_code = color
    return f'{TerminalColor.BOLD}{color_code}{text}{TerminalColor.END_COLOR}'


if __name__ == '__main__':
    reg_expr = ProjectPostgresRegExp.PRO
    do_cleanup(reg_expr)



import json
from datetime import datetime
from pathlib import Path

import pytest


@pytest.fixture(scope='session')
def output_folder():
    folder = Path(__file__).parent.parent / 'output'
    return folder


@pytest.fixture(scope='session')
def fixtures_folder():
    folder = Path(__file__).parent.parent / 'tests' / 'fixtures'
    return folder


@pytest.fixture(scope='session')
def camel_case_dict():
    data = {
        "firstName": "Luis",
        "lastName": "Wayne",
        "IMEI": "HJ8777"
    }
    return data


@pytest.fixture(scope='session')
def model_list_for_serializers():
    with open('./fixtures/model_list.json', 'r') as json_file:
        model_list = json.load(json_file)
    return model_list


@pytest.fixture()
def d_local_data_dict():
    data = {
        "id": "D-4-be8eda8c-5fe7-49dd-8058-4ddaac00611b",
        "amount": 72.00,
        "status": "PAID",
        "status_detail": "The payment was paid.",
        "status_code": "200",
        "currency": "USD",
        "country": "AR",
        "payment_method_id": "RP",
        "payment_method_type": "TICKET",
        "payment_method_flow": "REDIRECT",
        "payer": {
            "name": "Nino Deicas",
            "user_reference": "US-jmh3gb4kj5h34",
            "email": "buyer@gmail.com",
            "address": {
                "street": "123th street",
                "state": "FL",
                "zip_code": "99999999"
            }
        },
        "order_id": "4m1OdghPUQtg",
        "notification_url": "http://www.merchant.com/notifications",
        "created_date": "2019-06-26T15:17:31.000+0000",
        "user": {
            "id": 1,
            "username": "bwayne",
            "created": datetime(2022, 10, 21, 11, 56, 25, 23)
        }
    }
    return data




from decimal import Decimal
from typing import List, Union

from pydantic import BaseModel, Field


class APIResponse(BaseModel):
    '''
    Not used with responses for BCP so that we can use the __dict__ attribute without issues
    '''
    status_code: int
    message: str

    class Config:
        allow_population_by_field_name = True
        arbitrary_types_allowed = True


class PaymentAmount(BaseModel):
    amount_type: str = Field(alias='amountType')
    amount: str

    class Config:
        allow_population_by_field_name = True
        arbitrary_types_allowed = True


class PaymentDocument(BaseModel):
    '''
    Document ID must identify a payment uniquely as it will be used as a key
    in the payment endpoint.
    For now, we are using: {ContractId}-{shouldBeRepaidOn}
    This respects the restriction that document id must be at most 28 characters long.
    shouldBeRepaidOn comes from Zoral in the format %m-%d-%Y.
    We convert it and use it in the format: %Y-%m-%d for internal use and for the id.
    '''
    document_id: str = Field(alias='documentId')
    expiration_date: str = Field(alias='expirationDate')
    payment_detail: str = Field(alias='paymentDetail')
    amounts: List[PaymentAmount]

    class Config:
        allow_population_by_field_name = True
        arbitrary_types_allowed = True


class ClientLookupResponse(BaseModel):
    rqUUID: str
    result_code: str = Field(alias='resultCode')
    result_description: str = Field(alias='resultDescription')
    operation_date: str = Field(alias='operationDate',
                                description='This is the time and date the transaction was processed')
    operation_number_company: Union[str, None] = Field(
        alias='operationNumberCompany',
        description='The field below follows the format:'
        ' LU-{ClientLookupTransaction.id} Where LU stands for Lookup'
    )
    customer_name: Union[str, None] = Field(alias='customerName')
    documents: Union[List[PaymentDocument], None]
    result_code_company: Union[str, None] = Field(alias='resultCodeCompany')
    result_description_company: Union[str, None] = Field(alias='resultDescriptionCompany')

    class Config:
        allow_population_by_field_name = True
        arbitrary_types_allowed = True


class ZoralPaymentSchedule(BaseModel):
    amount: Decimal
    should_be_repaid_on: str = Field(alias='shouldBeRepaidOn')

    class Config:
        allow_population_by_field_name = True
        arbitrary_types_allowed = True


class GraphQLPaymentScheduleResponse(APIResponse):
    data: List[ZoralPaymentSchedule]

    class Config:
        allow_population_by_field_name = True
        arbitrary_types_allowed = True


class ZoralClientBalance(BaseModel):
    contract_id: str = Field(alias='ContractId')
    first_name: str = Field(alias='firstname')
    last_name: str = Field(alias='lastname')
    next_payment_date: str = Field(alias='nextpaymentdate')
    loan_repayment_amount: Decimal = Field(alias='loanrepaymentamount')

    class Config:
        allow_population_by_field_name = True
        arbitrary_types_allowed = True


class GraphQLClientBalanceResponse(APIResponse):
    data: Union[ZoralClientBalance, None]

    class Config:
        allow_population_by_field_name = True
        arbitrary_types_allowed = True


class ClientDataLookupResult(BaseModel):
    successful: bool
    message: str
    customer_name: str
    financial_product_id: str
    documents: List[PaymentDocument]

    class Config:
        allow_population_by_field_name = True
        arbitrary_types_allowed = True


class PaymentResponse(BaseModel):
    rqUUID: str
    result_code: str = Field(alias='resultCode')
    result_description: str = Field(alias='resultDescription')
    operation_date: str = Field(alias='operationDate')
    operation_number_company: Union[str, None] = Field(alias='operationNumberCompany')
    endorsement: Union[str, None]
    result_code_company: Union[str, None] = Field(alias='resultCodeCompany')
    result_description_company: Union[str, None] = Field(alias='resultDescriptionCompany')

    class Config:
        allow_population_by_field_name = True
        arbitrary_types_allowed = True


class RefundResponse(BaseModel):
    rqUUID: str
    resultCode: str
    resultDescription: str
    operationDate: str
    result_code_company: Union[str, None] = Field(alias='resultCodeCompany')
    result_description_company: Union[str, None] = Field(alias='resultDescriptionCompany')

    class Config:
        allow_population_by_field_name = True
        arbitrary_types_allowed = True



from datetime import datetime, date
from typing import Dict, Any, List
from typing import NamedTuple


class AssertionTuple(NamedTuple):
    var_name: str
    value: Any


def process_dict(data: Dict[str, Any], var_name: str) -> List[AssertionTuple]:
    assertion_list = list()
    for key, value in data.items():
        var_name_dict = f'{var_name}[\'{key}\']'
        if isinstance(value, dict):
            processed_list = process_dict(value, var_name_dict)
            assertion_list.extend(processed_list)
        else:
            processor_function = PROCESSOR_FUNCTIONS.get(type(value))
            if processor_function is not None:
                processed_list = processor_function(value, var_name_dict)
                assertion_list.extend(processed_list)

    return assertion_list


def process_datetime(data: datetime, var_name: str) -> List[AssertionTuple]:
    value = f'datetime({data.year}, {data.month}, {data.day}, {data.hour}, {data.minute},' \
            f' {data.second}, {data.microsecond})'

    return [AssertionTuple(var_name=var_name, value=value)]


def process_date(data: date, var_name: str) -> List[AssertionTuple]:
    value = f'date({data.year}, {data.month}, {data.day})'
    return [AssertionTuple(var_name=var_name, value=value)]


def process_str(data: str, var_name: str) -> List[AssertionTuple]:
    assertion_list = list()
    assertion_line = AssertionTuple(var_name=var_name, value=f"'{data}'")
    assertion_list.append(assertion_line)
    return assertion_list


def process_raw(data: Any, var_name: str) -> List[AssertionTuple]:
    return [AssertionTuple(var_name=var_name, value=data)]


def process_list(data: List[Any], var_name: str) -> List[AssertionTuple]:
    assertion_list = list()

    for i, value in enumerate(data):
        new_var_name = f'{var_name}[{i}]'
        processor_function = PROCESSOR_FUNCTIONS.get(type(value))
        if processor_function is not None:
            processed_list = processor_function(value, new_var_name)
            if len(processed_list) > 0:
                assertion_list.extend(processed_list)
    return assertion_list


PROCESSOR_FUNCTIONS = {
    datetime: process_datetime,
    date: process_date,
    dict: process_dict,
    str: process_str,
    int: process_raw,
    bool: process_raw,
    float: process_raw,
    list: process_list
}


def generate_assertion_tuples(data_dict: Dict[str, Any], var_name: str) -> List[AssertionTuple]:
    assertion_list = list()
    for key, value in data_dict.items():
        new_var_name = f'{var_name}[\'{key}\']'
        processor_function = PROCESSOR_FUNCTIONS.get(type(value))
        if processor_function is not None:
            processed_list = processor_function(value, new_var_name)
            if len(processed_list) > 0:
                assertion_list.extend(processed_list)
    return assertion_list


def generate_pytest_assertions(data_dict: Dict[str, Any], var_name: str) -> List[str]:
    pytest_assertion_list = list()
    assertion_list: List[AssertionTuple] = generate_assertion_tuples(data_dict, var_name)
    for assertion in assertion_list:
        if isinstance(assertion.value, bool):
            if assertion.value:
                assertion_message = f'assert {assertion.var_name}'
            else:
                assertion_message = f'assert not {assertion.var_name}'
        else:
            assertion_message = f'assert {assertion.var_name} == {assertion.value}'
        print(assertion_message)
        pytest_assertion_list.append(assertion_message)
    return pytest_assertion_list



import base64
import warnings
from datetime import datetime, timedelta
from typing import Optional, List, Any, Dict, Union

from pydantic import BaseModel, Field, root_validator

from .. import __version__ as sdk_version
from ..enums import Countries, Currencies
from ..models import Payer
from ..utils import convert_to_iso8601


class M3Headers(BaseModel):
    """This headers are modeled based on this documentation
     https://payjoy.atlassian.net/wiki/spaces/ENGINEERIN/pages/2690619670541/Finance+Engine+API+Webhook+-+Successful+payment+notification
    """
    content_type: str = Field(alias='Content-Type', default='application/json')
    user_agent: str = Field(alias='User-Agent', default=f'Payjoy D-Local SDK / {sdk_version}')
    authorization: Optional[str] = Field(alias='Authorization')

    def set_authorization(self, username: str, password: str, encoding: str = 'utf-8') -> str:
        base64_encoded: str = base64.b64encode(f'{username}:{password}'.encode(encoding)).decode(encoding)
        self.authorization = f'Basic {base64_encoded}'
        return self.authorization


class M3Payment(BaseModel):
    """Payload content for Payment resolution after D-Local calls the notification_url."""
    order_id: str = Field(..., alias='paymentId')
    finance_engine_version: str = Field(..., alias='financeEngineVersion')
    credit_line_id: str = Field(..., alias='creditLineId')
    payment_provider: str = Field(..., alias='paymentProvider')
    country: Countries
    currency: Currencies
    amount: float
    timestamp_paid: str = Field(..., alias='timestampPaid')
    device_id: str = Field(..., alias='deviceId')
    user_id: str = Field(..., alias='userId')
    merchant_id: str = Field(..., alias='merchantId')
    clerk_id: str = Field(..., alias='clerkId')
    final_payment_info: Any = Field(..., alias='finalPaymentInfo')

    class Config:
        allow_population_by_field_name = True


class M3PaymentResolutionRequest(BaseModel):
    payments: List[M3Payment]

    class Config:
        allow_population_by_field_name = True


class M3PaymentResolutionResponse(BaseModel):
    valid: bool
    message: str


class M3PaymentRequest(BaseModel):
    """Payload sent to Payment's microservice from M3 to be sent to D-Local
    M3 ----M3PaymentRequest ----> Payments
    M3 <---M3PaymentResponse ---- Payments
    """
    amount: Union[int, float] = Field(description='Amount paid')
    payer: Payer = Field(description='Payer for the payment.')
    credit_line_id: str = Field(description='Unique identifier for a credit line or a finance order.')
    finance_engine_version: str = Field(description='Finance engine version FEv1 or FEv2')
    country: Countries = Field(description='Two letter country code of the country the device was sold in.')
    currency: Currencies = Field(description='The three letter currency code abbreviation.')
    payment_provider: str = Field(description='Enumeration value for the payment provider, ie D_LOCAL.')
    device_id: str = Field(description='This will contain the foreign key to the Devices table')
    user_id: str = Field(description='Foreign key to the Users table in M2 or the Persons table in M3')
    merchant_id: str = Field(description='Reference to the Merchants table. '
                                         'ONLY FEv2 needs to publish this information')
    clerk_id: str = Field(description='Foreign key to the clerk who sold the device')

    def build_payment_resolution_request(self, timestamp_paid: str, order_id: str) -> M3Payment:
        m3_payment_dict = self.dict()
        m3_payment_dict['timestamp_paid'] = timestamp_paid
        m3_payment_dict['final_payment_info'] = None
        m3_payment_dict['order_id'] = order_id

        m3_payment_request = M3Payment(**m3_payment_dict)
        return m3_payment_request


class M3PaymentResponse(BaseModel):
    """Response from Payments to M3 after D-Local responded
    M3 ----M3PaymentRequest ----> Payments
    M3 <---M3PaymentResponse ---- Payments
    """
    order_id: str = Field(description='Unique UUID generated by the Payment\'s microservice.')
    redirect_url: str = Field(description='Url to redirect reported by D-Local')
    status: str = Field(description='Status of the payment reported by D-Local')
    amount: float = Field(description='Amount paid')
    payments_created: datetime = Field(description='Date and time the payment was '
                                                   'created in the Payment\'s microservice.')
    date_received: datetime = Field(description='Date and time reported by D-Local as created.')
    expiration_time: Optional[datetime] = Field(description='Date and time that the ticket expires.')

    # _expiration_delta: Optional[timedelta] = PrivateAttr(default=timedelta(days=5))

    class Config:
        json_encoders = {
            datetime: lambda x: convert_to_iso8601(x)
        }

    @root_validator
    def compute_expiration_time(cls, values) -> Dict:
        """This is not the intended use of the @root_validator, but it was the cleanest way to include the
        expiration_time in the dictionary payload."""
        # TODO use _expiration_delta instead of hard coding 5 days.
        new_expiration_time = values['date_received'] + timedelta(days=5)
        if values['expiration_time'] is not None:
            warnings.warn(f'Overriding expiration_time with {new_expiration_time}')
        values['expiration_time'] = new_expiration_time
        return values



import re
from functools import partial
from typing import List, Pattern

from django_scaffolding_tools.enums import PatternType


def get_pattern_type(value: str, patterns: List[Pattern], expected_pattern: PatternType) -> Pattern:
    for pattern in patterns:
        if pattern.match(value):
            return expected_pattern


EMAIL_PATTERNS = [
    re.compile(
        r'^([a-zA-Z0-9_\-\.]+)@((\[[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.)|(([a-zA-Z0-9\-]+\.)+))'
        r'([a-zA-Z]{2,4}|[0-9]{1,3})(\]?)$')
]
URL_PATTERNS = [
    re.compile(
        r'^https?:\/\/(?:www\.)?[-a-zA-Z0-9@:%._\+~#=]{1,256}\.[a-zA-Z0-9()]{1,6}\b(?:[-a-zA-Z0-9()@:%_\+.~#?&\/=]*)')
]

get_email_pattern = partial(get_pattern_type, patterns=EMAIL_PATTERNS, expected_pattern=PatternType.EMAIL)
get_url_pattern = partial(get_pattern_type, patterns=URL_PATTERNS, expected_pattern=PatternType.URL)

PATTERN_FUNCTIONS = [get_email_pattern, get_url_pattern]



import os
import smtplib
from email import encoders
from email.mime.base import MIMEBase
from email.mime.multipart import MIMEMultipart
from email.mime.text import MIMEText

from django_scaffolding_tools._experimental.cover_letters.config import ConfigurationManager


def send_email(receiver_address, sender_address, password, body, subject, attachments, **kwargs):
    # The mail addresses and password

    # receiver_address = 'lberrocal@pancanal.com'
    # Setup the MIME
    message = MIMEMultipart()
    message['From'] = sender_address
    message['To'] = receiver_address
    message['Subject'] = subject
    # The subject line
    # The body and the attachments for the mail
    message.attach(MIMEText(body, 'plain'))
    for attachment in attachments:
        attach_file = open(attachment, 'rb')  # Open the file as binary mode
        payload = MIMEBase('application', 'octate-stream')
        payload.set_payload((attach_file).read())
        encoders.encode_base64(payload)  # encode the attachment
        # add payload header with filename
        # payload.add_header('Content-Decomposition', 'attachment', filename='invoices_20201101_1120.xlsx')
        fn = os.path.basename(attachment)
        payload.add_header(
            "Content-Disposition",
            f"attachment; filename= {fn}",
        )
        message.attach(payload)
    # Create SMTP session for sending the mail
    session = smtplib.SMTP('smtp.gmail.com', 587)  # use gmail with port
    session.starttls()  # enable security
    session.login(sender_address, password)  # login with mail_id and password
    text = message.as_string()
    session.sendmail(sender_address, receiver_address, text)
    session.quit()
    print('Mail Sent')


def send_cover_letter(receiver_email, context, attachments):
    config = ConfigurationManager().get_current()
    email = config['gmail']['email']
    token = config['gmail']['token']

    body = f"""Dear Hiring Manager:

    I would like to be considered for the position {context['position_name']} at {context["company_name"]}.

    Enclosed you'll find my cover letter and cv.

    Regards

    Luis
    """
    subject = f'Application for {context["position_name"]}.'
    send_email(receiver_email, email, token, body, subject, attachments)


if __name__ == '__main__':
    attachment_file = '/home/luiscberrocal/PycharmProjects/django_scaffolding_tools' \
                      '/output/20221107_cover_Jedi Order Council_Jedi Knight.pdf'
    to_email = 'laberrocalf@gmail.com'
    context = {'position_name': 'Jedi knight', 'company_name': 'Jedi council'}
    send_cover_letter(to_email, context, [attachment_file])



from django_scaffolding_tools.parsers import parse_file_for_ast_classes
from django_scaffolding_tools.utils.core import quick_write


def test_parse_pydantic(fixtures_folder):
    pydantic_file = fixtures_folder / 'data_classes.py'
    ast_dict = parse_file_for_ast_classes(pydantic_file)
    write_intermediate = True
    if write_intermediate:
        model_filename = 'dc'
        quick_write(ast_dict, f'ast_{model_filename}.json', output_subfolder='pydantic')



import os

from django_scaffolding_tools.django.utils import DjangoAppManager


def test_django_loading():
    os.environ.setdefault("DJANGO_SETTINGS_MODULE", ".settings")
    import django
    django.setup()
    manager = DjangoAppManager()
    print(list(manager.get_installed_apps()))



from django_scaffolding_tools.django.parsers import parse_for_django_classes
from django_scaffolding_tools.parsers import parse_file_for_ast_classes
from django_scaffolding_tools.utils.assert_utils import generate_pytest_assertions
from django_scaffolding_tools.utils.core import quick_write


def test_class_list(fixtures_folder, output_folder):
    module_file = 'models_with_helptext.py'
    filename = fixtures_folder / module_file

    ast_module = parse_file_for_ast_classes(filename)
    # quick_write(ast_module, f'ast_{module_file}.json', output_subfolder='django')

    django_classes = parse_for_django_classes(ast_module)
    # generate_pytest_assertions(django_classes, 'django_classes')
    # quick_write(django_classes, f'classes_{module_file}.json', output_subfolder='django')

    assert django_classes['classes'][0]['name'] == 'Payer'
    attrs_0 = django_classes['classes'][0]['attributes']
    assert attrs_0[0]['name'] == 'name'
    assert attrs_0[0]['keywords'][0]['name'] == 'max_length'
    assert attrs_0[0]['keywords'][0]['value_type_TMP'] == 'Constant'
    assert attrs_0[0]['keywords'][0]['value'] == 100
    assert attrs_0[0]['data_type'] == 'CharField'
    assert attrs_0[1]['name'] == 'email'
    assert attrs_0[1]['keywords'][0]['name'] == 'max_length'
    assert attrs_0[1]['keywords'][0]['value_type_TMP'] == 'Constant'
    assert attrs_0[1]['keywords'][0]['value'] == 100
    assert attrs_0[1]['data_type'] == 'CharField'
    assert attrs_0[2]['name'] == 'document'
    assert attrs_0[2]['keywords'][0]['name'] == 'max_length'
    assert attrs_0[2]['keywords'][0]['value_type_TMP'] == 'Constant'
    assert attrs_0[2]['keywords'][0]['value'] == 100
    assert attrs_0[2]['keywords'][1]['name'] == 'help_text'
    assert attrs_0[2]['keywords'][1]['value_type_TMP'] == 'Call'
    assert attrs_0[2]['keywords'][1]['value'] == 'User’s personal identification number. To see the document ' \
                                                 'code list per country, go to the Country Reference page.'
    assert attrs_0[2]['data_type'] == 'CharField'
    assert attrs_0[3]['name'] == 'country'
    assert attrs_0[3]['keywords'][0]['name'] == 'max_length'
    assert attrs_0[3]['keywords'][0]['value_type_TMP'] == 'Constant'
    assert attrs_0[3]['keywords'][0]['value'] == 2
    assert attrs_0[3]['keywords'][1]['name'] == 'help_text'
    assert attrs_0[3]['keywords'][1]['value_type_TMP'] == 'Call'
    assert attrs_0[3]['keywords'][1][
               'value'] == 'This will contain the two letter country code of the country the device was sold in.'
    assert attrs_0[3]['data_type'] == 'CharField'

    assert django_classes['classes'][1]['name'] == 'Payment'
    assert django_classes['classes'][1]['attributes'][0]['name'] == 'payer'
    assert django_classes['classes'][1]['attributes'][0]['keywords'][0]['name'] == 'verbose_name'
    assert django_classes['classes'][1]['attributes'][0]['keywords'][0]['value_type_TMP'] == 'Call'
    assert django_classes['classes'][1]['attributes'][0]['keywords'][0][
               'value'] == 'This will contain the two letter country code of the country the device was sold in.'
    assert django_classes['classes'][1]['attributes'][0]['keywords'][1]['name'] == 'on_delete'
    assert django_classes['classes'][1]['attributes'][0]['keywords'][1]['value_type_TMP'] == 'Attribute'
    assert django_classes['classes'][1]['attributes'][0]['keywords'][1]['value'] == 'PROTECT'
    assert django_classes['classes'][1]['attributes'][0]['keywords'][2]['name'] == 'related_name'
    assert django_classes['classes'][1]['attributes'][0]['keywords'][2]['value_type_TMP'] == 'Constant'
    assert django_classes['classes'][1]['attributes'][0]['keywords'][2]['value'] == 'payments'
    assert django_classes['classes'][1]['attributes'][0]['data_type'] == 'ForeignKey'
    assert django_classes['classes'][1]['attributes'][1]['name'] == 'credit_line_id'
    assert django_classes['classes'][1]['attributes'][1]['keywords'][0]['name'] == 'max_length'
    assert django_classes['classes'][1]['attributes'][1]['keywords'][0]['value_type_TMP'] == 'Constant'
    assert django_classes['classes'][1]['attributes'][1]['keywords'][0]['value'] == 64
    assert django_classes['classes'][1]['attributes'][1]['keywords'][1]['name'] == 'help_text'
    assert django_classes['classes'][1]['attributes'][1]['keywords'][1]['value_type_TMP'] == 'Call'
    assert django_classes['classes'][1]['attributes'][1]['keywords'][1][
               'value'] == 'Unique identifier for a credit line or a finance order.'
    assert django_classes['classes'][1]['attributes'][1]['data_type'] == 'CharField'
    assert django_classes['classes'][1]['attributes'][2]['name'] == 'finance_engine_version'
    assert django_classes['classes'][1]['attributes'][2]['keywords'][0]['name'] == 'max_length'
    assert django_classes['classes'][1]['attributes'][2]['keywords'][0]['value_type_TMP'] == 'Constant'
    assert django_classes['classes'][1]['attributes'][2]['keywords'][0]['value'] == 5
    assert django_classes['classes'][1]['attributes'][2]['keywords'][1]['name'] == 'help_text'
    assert django_classes['classes'][1]['attributes'][2]['keywords'][1]['value_type_TMP'] == 'Call'
    assert django_classes['classes'][1]['attributes'][2]['keywords'][1][
               'value'] == 'Finance engine version FEv1 or FEv2'
    assert django_classes['classes'][1]['attributes'][2]['data_type'] == 'CharField'
    assert django_classes['classes'][1]['attributes'][3]['name'] == 'user_id'
    assert django_classes['classes'][1]['attributes'][3]['keywords'][0]['name'] == 'max_length'
    assert django_classes['classes'][1]['attributes'][3]['keywords'][0]['value_type_TMP'] == 'Constant'
    assert django_classes['classes'][1]['attributes'][3]['keywords'][0]['value'] == 50
    assert django_classes['classes'][1]['attributes'][3]['keywords'][1]['name'] == 'help_text'
    assert django_classes['classes'][1]['attributes'][3]['keywords'][1]['value_type_TMP'] == 'Call'
    assert django_classes['classes'][1]['attributes'][3]['keywords'][1][
               'value'] == 'This will contain the foreign key to the Users table in M2 or the Persons table in M3'
    assert django_classes['classes'][1]['attributes'][3]['data_type'] == 'CharField'
    assert django_classes['classes'][1]['attributes'][4]['name'] == 'device_id'
    assert django_classes['classes'][1]['attributes'][4]['keywords'][0]['name'] == 'max_length'
    assert django_classes['classes'][1]['attributes'][4]['keywords'][0]['value_type_TMP'] == 'Constant'
    assert django_classes['classes'][1]['attributes'][4]['keywords'][0]['value'] == 50
    assert django_classes['classes'][1]['attributes'][4]['keywords'][1]['name'] == 'help_text'
    assert django_classes['classes'][1]['attributes'][4]['keywords'][1]['value_type_TMP'] == 'Call'
    assert django_classes['classes'][1]['attributes'][4]['keywords'][1][
               'value'] == 'This will contain the foreign key to the Devices table'
    assert django_classes['classes'][1]['attributes'][4]['data_type'] == 'CharField'
    assert django_classes['classes'][1]['attributes'][5]['name'] == 'merchant_id'
    assert django_classes['classes'][1]['attributes'][5]['keywords'][0]['name'] == 'max_length'
    assert django_classes['classes'][1]['attributes'][5]['keywords'][0]['value_type_TMP'] == 'Constant'
    assert django_classes['classes'][1]['attributes'][5]['keywords'][0]['value'] == 64
    assert django_classes['classes'][1]['attributes'][5]['keywords'][1]['name'] == 'null'
    assert django_classes['classes'][1]['attributes'][5]['keywords'][1]['value_type_TMP'] == 'Constant'
    assert django_classes['classes'][1]['attributes'][5]['keywords'][1]['value']
    assert django_classes['classes'][1]['attributes'][5]['keywords'][2]['name'] == 'blank'
    assert django_classes['classes'][1]['attributes'][5]['keywords'][2]['value_type_TMP'] == 'Constant'
    assert django_classes['classes'][1]['attributes'][5]['keywords'][2]['value']
    assert django_classes['classes'][1]['attributes'][5]['keywords'][3]['name'] == 'help_text'
    assert django_classes['classes'][1]['attributes'][5]['keywords'][3]['value_type_TMP'] == 'Call'
    assert django_classes['classes'][1]['attributes'][5]['keywords'][3][
               'value'] == 'Reference to the Merchants table - ONLY FEv2 needs to publish this information'
    assert django_classes['classes'][1]['attributes'][5]['data_type'] == 'CharField'
    assert django_classes['classes'][1]['attributes'][6]['name'] == 'clerk_id'
    assert django_classes['classes'][1]['attributes'][6]['keywords'][0]['name'] == 'max_length'
    assert django_classes['classes'][1]['attributes'][6]['keywords'][0]['value_type_TMP'] == 'Constant'
    assert django_classes['classes'][1]['attributes'][6]['keywords'][0]['value'] == 50
    assert django_classes['classes'][1]['attributes'][6]['keywords'][1]['name'] == 'null'
    assert django_classes['classes'][1]['attributes'][6]['keywords'][1]['value_type_TMP'] == 'Constant'
    assert django_classes['classes'][1]['attributes'][6]['keywords'][1]['value']
    assert django_classes['classes'][1]['attributes'][6]['keywords'][2]['name'] == 'blank'
    assert django_classes['classes'][1]['attributes'][6]['keywords'][2]['value_type_TMP'] == 'Constant'
    assert django_classes['classes'][1]['attributes'][6]['keywords'][2]['value']
    assert django_classes['classes'][1]['attributes'][6]['keywords'][3]['name'] == 'help_text'
    assert django_classes['classes'][1]['attributes'][6]['keywords'][3]['value_type_TMP'] == 'Call'
    assert django_classes['classes'][1]['attributes'][6]['keywords'][3][
               'value'] == 'This will contain the foreign key to the clerk who sold the device'
    assert django_classes['classes'][1]['attributes'][6]['data_type'] == 'CharField'
    assert django_classes['classes'][1]['attributes'][7]['name'] == 'finance_provider'
    assert django_classes['classes'][1]['attributes'][7]['keywords'][0]['name'] == 'max_length'
    assert django_classes['classes'][1]['attributes'][7]['keywords'][0]['value_type_TMP'] == 'Constant'
    assert django_classes['classes'][1]['attributes'][7]['keywords'][0]['value'] == 50
    assert django_classes['classes'][1]['attributes'][7]['keywords'][1]['name'] == 'help_text'
    assert django_classes['classes'][1]['attributes'][7]['keywords'][1]['value_type_TMP'] == 'Call'
    assert django_classes['classes'][1]['attributes'][7]['keywords'][1][
               'value'] == 'Constants representing the finance provider.'
    assert django_classes['classes'][1]['attributes'][7]['data_type'] == 'CharField'
    assert django_classes['classes'][1]['attributes'][8]['name'] == 'payment_provider'
    assert django_classes['classes'][1]['attributes'][8]['keywords'][0]['name'] == 'max_length'
    assert django_classes['classes'][1]['attributes'][8]['keywords'][0]['value_type_TMP'] == 'Constant'
    assert django_classes['classes'][1]['attributes'][8]['keywords'][0]['value'] == 16
    assert django_classes['classes'][1]['attributes'][8]['data_type'] == 'CharField'
    assert django_classes['classes'][1]['attributes'][9]['name'] == 'amount'
    assert django_classes['classes'][1]['attributes'][9]['keywords'][0]['name'] == 'max_digits'
    assert django_classes['classes'][1]['attributes'][9]['keywords'][0]['value_type_TMP'] == 'Constant'
    assert django_classes['classes'][1]['attributes'][9]['keywords'][0]['value'] == 12
    assert django_classes['classes'][1]['attributes'][9]['keywords'][1]['name'] == 'decimal_places'
    assert django_classes['classes'][1]['attributes'][9]['keywords'][1]['value_type_TMP'] == 'Constant'
    assert django_classes['classes'][1]['attributes'][9]['keywords'][1]['value'] == 2
    assert django_classes['classes'][1]['attributes'][9]['data_type'] == 'DecimalField'
    assert django_classes['classes'][1]['attributes'][10]['name'] == 'currency'
    assert django_classes['classes'][1]['attributes'][10]['keywords'][0]['name'] == 'max_length'
    assert django_classes['classes'][1]['attributes'][10]['keywords'][0]['value_type_TMP'] == 'Constant'
    assert django_classes['classes'][1]['attributes'][10]['keywords'][0]['value'] == 3
    assert django_classes['classes'][1]['attributes'][10]['keywords'][1]['name'] == 'help_text'
    assert django_classes['classes'][1]['attributes'][10]['keywords'][1]['value_type_TMP'] == 'Call'
    assert django_classes['classes'][1]['attributes'][10]['keywords'][1][
               'value'] == 'The three letter currency code abbreviation.'
    assert django_classes['classes'][1]['attributes'][10]['data_type'] == 'CharField'
    assert django_classes['classes'][1]['attributes'][11]['name'] == 'country'
    assert django_classes['classes'][1]['attributes'][11]['keywords'][0]['name'] == 'max_length'
    assert django_classes['classes'][1]['attributes'][11]['keywords'][0]['value_type_TMP'] == 'Constant'
    assert django_classes['classes'][1]['attributes'][11]['keywords'][0]['value'] == 2
    assert django_classes['classes'][1]['attributes'][11]['keywords'][1]['name'] == 'help_text'
    assert django_classes['classes'][1]['attributes'][11]['keywords'][1]['value_type_TMP'] == 'Call'
    assert django_classes['classes'][1]['attributes'][11]['keywords'][1][
               'value'] == 'This will contain the two letter country code of the country the device was sold in.'
    assert django_classes['classes'][1]['attributes'][11]['data_type'] == 'CharField'
    assert django_classes['classes'][1]['attributes'][12]['name'] == 'payment_method_id'
    assert django_classes['classes'][1]['attributes'][12]['keywords'][0]['name'] == 'max_length'
    assert django_classes['classes'][1]['attributes'][12]['keywords'][0]['value_type_TMP'] == 'Constant'
    assert django_classes['classes'][1]['attributes'][12]['keywords'][0]['value'] == 3
    assert django_classes['classes'][1]['attributes'][12]['keywords'][1]['name'] == 'help_text'
    assert django_classes['classes'][1]['attributes'][12]['keywords'][1]['value_type_TMP'] == 'Call'
    assert django_classes['classes'][1]['attributes'][12]['keywords'][1][
               'value'] == 'Payment method defined by D-Local.'
    assert django_classes['classes'][1]['attributes'][12]['data_type'] == 'CharField'
    assert django_classes['classes'][1]['attributes'][13]['name'] == 'payment_method_flow'
    assert django_classes['classes'][1]['attributes'][13]['keywords'][0]['name'] == 'max_length'
    assert django_classes['classes'][1]['attributes'][13]['keywords'][0]['value_type_TMP'] == 'Constant'
    assert django_classes['classes'][1]['attributes'][13]['keywords'][0]['value'] == 15
    assert django_classes['classes'][1]['attributes'][13]['keywords'][1]['name'] == 'help_text'
    assert django_classes['classes'][1]['attributes'][13]['keywords'][1]['value_type_TMP'] == 'Call'
    assert django_classes['classes'][1]['attributes'][13]['keywords'][1][
               'value'] == 'D-Local payment flow usually DIRECT or REDIRECT.'
    assert django_classes['classes'][1]['attributes'][13]['data_type'] == 'CharField'
    assert django_classes['classes'][1]['attributes'][14]['name'] == 'order_id'
    assert django_classes['classes'][1]['attributes'][14]['keywords'][0]['name'] == 'primary_key'
    assert django_classes['classes'][1]['attributes'][14]['keywords'][0]['value_type_TMP'] == 'Constant'
    assert not django_classes['classes'][1]['attributes'][14]['keywords'][0]['value']
    assert django_classes['classes'][1]['attributes'][14]['keywords'][1]['name'] == 'unique'
    assert django_classes['classes'][1]['attributes'][14]['keywords'][1]['value_type_TMP'] == 'Constant'
    assert django_classes['classes'][1]['attributes'][14]['keywords'][1]['value']
    assert django_classes['classes'][1]['attributes'][14]['keywords'][2]['name'] == 'default'
    assert django_classes['classes'][1]['attributes'][14]['keywords'][2]['value_type_TMP'] == 'Attribute'
    assert django_classes['classes'][1]['attributes'][14]['keywords'][2]['value'] == 'uuid4'
    assert django_classes['classes'][1]['attributes'][14]['keywords'][3]['name'] == 'editable'
    assert django_classes['classes'][1]['attributes'][14]['keywords'][3]['value_type_TMP'] == 'Constant'
    assert not django_classes['classes'][1]['attributes'][14]['keywords'][3]['value']
    assert django_classes['classes'][1]['attributes'][14]['data_type'] == 'UUIDField'
    assert django_classes['classes'][1]['attributes'][15]['name'] == 'notification_url'
    assert django_classes['classes'][1]['attributes'][15]['data_type'] == 'URLField'
    assert django_classes['classes'][1]['attributes'][16]['name'] == 'payment_status'
    assert django_classes['classes'][1]['attributes'][16]['keywords'][0]['name'] == 'max_length'
    assert django_classes['classes'][1]['attributes'][16]['keywords'][0]['value_type_TMP'] == 'Constant'
    assert django_classes['classes'][1]['attributes'][16]['keywords'][0]['value'] == 15
    assert django_classes['classes'][1]['attributes'][16]['keywords'][1]['name'] == 'null'
    assert django_classes['classes'][1]['attributes'][16]['keywords'][1]['value_type_TMP'] == 'Constant'
    assert django_classes['classes'][1]['attributes'][16]['keywords'][1]['value']
    assert django_classes['classes'][1]['attributes'][16]['keywords'][2]['name'] == 'blank'
    assert django_classes['classes'][1]['attributes'][16]['keywords'][2]['value_type_TMP'] == 'Constant'
    assert django_classes['classes'][1]['attributes'][16]['keywords'][2]['value']
    assert django_classes['classes'][1]['attributes'][16]['keywords'][3]['name'] == 'help_text'
    assert django_classes['classes'][1]['attributes'][16]['keywords'][3]['value_type_TMP'] == 'Call'
    assert django_classes['classes'][1]['attributes'][16]['keywords'][3][
               'value'] == 'The values should match pj_d_local_sdk.enums.PaymentStatus values.'
    assert django_classes['classes'][1]['attributes'][16]['data_type'] == 'CharField'
    assert django_classes['classes'][1]['attributes'][17]['name'] == 'response'
    assert django_classes['classes'][1]['attributes'][17]['keywords'][0]['name'] == 'default'
    assert django_classes['classes'][1]['attributes'][17]['keywords'][0]['value_type_TMP'] == 'Name'
    assert django_classes['classes'][1]['attributes'][17]['keywords'][0]['value'] == 'get_response_default_dict'
    assert django_classes['classes'][1]['attributes'][17]['keywords'][1]['name'] == 'help_text'
    assert django_classes['classes'][1]['attributes'][17]['keywords'][1]['value_type_TMP'] == 'Call'
    assert django_classes['classes'][1]['attributes'][17]['keywords'][1][
               'value'] == 'Response from D-Local API. Will contain 2 keys for request response and callback response'
    assert django_classes['classes'][1]['attributes'][17]['data_type'] == 'JSONField'
    assert django_classes['classes'][1]['attributes'][18]['name'] == 'state'
    assert django_classes['classes'][1]['attributes'][18]['keywords'][0]['name'] == 'protected'
    assert django_classes['classes'][1]['attributes'][18]['keywords'][0]['value_type_TMP'] == 'Constant'
    assert django_classes['classes'][1]['attributes'][18]['keywords'][0]['value']
    assert django_classes['classes'][1]['attributes'][18]['keywords'][1]['name'] == 'default'
    assert django_classes['classes'][1]['attributes'][18]['keywords'][1]['value_type_TMP'] == 'Attribute'
    assert django_classes['classes'][1]['attributes'][18]['keywords'][1]['value'] == 'CREATED'
    assert django_classes['classes'][1]['attributes'][18]['data_type'] == 'FSMField'
    assert django_classes['classes'][1]['attributes'][19]['name'] == 'metadata'
    assert django_classes['classes'][1]['attributes'][19]['keywords'][0]['name'] == 'default'
    assert django_classes['classes'][1]['attributes'][19]['keywords'][0]['value_type_TMP'] == 'Name'
    assert django_classes['classes'][1]['attributes'][19]['keywords'][0]['value'] == 'get_metadata_default_dict'
    assert django_classes['classes'][1]['attributes'][19]['keywords'][1]['name'] == 'help_text'
    assert django_classes['classes'][1]['attributes'][19]['keywords'][1]['value_type_TMP'] == 'Call'
    assert django_classes['classes'][1]['attributes'][19]['keywords'][1][
               'value'] == 'Metadata for the payment. It includes the dates with the changes of state.'
    assert django_classes['classes'][1]['attributes'][19]['data_type'] == 'JSONField'


def test_class_list_finance(fixtures_folder, output_folder):
    module_file = 'finance_models.py'
    filename = fixtures_folder / module_file

    ast_module = parse_file_for_ast_classes(filename)
    quick_write(ast_module, f'ast_{module_file}.json', output_subfolder='django')

    django_classes = parse_for_django_classes(ast_module)
    generate_pytest_assertions(django_classes, 'django_classes')
    quick_write(django_classes, f'classes_{module_file}.json', output_subfolder='django')



from datetime import datetime, date

from django_scaffolding_tools.utils.assert_utils import generate_assertion_tuples


def test_generate_assert_dict():
    data_dict = {'name': 'Bruce', 'date_of_birth': datetime(1966, 5, 5, 13, 15, 0),
                 'pets': [{'type': 'dog', 'name': 'Nanook', 'dob': date(2019, 8, 5)}],
                 'address': {'street': 'First street', 'zip_code': '999999', 'super_hero': True, 'power': 7}}
    # data_dict = {'name': 'Bruce'}
    assert_list = generate_assertion_tuples(data_dict, 'data_dict')
    for assertion_tuple in assert_list:
        print(f'{assertion_tuple.var_name} == {assertion_tuple.value}')



from decimal import Decimal, ROUND_HALF_UP
import logging
from django.conf import settings
from django.core.exceptions import ValidationError, NON_FIELD_ERRORS
from django.db import models, IntegrityError, transaction
from django.db.models import Sum, Q
from django.utils import timezone
from django.utils.translation import ugettext_lazy as _
# Create your models here.
from model_utils.models import TimeStampedModel

from .exceptions import FinanceException
from .managers import InvoiceManager, EncounterManager, ClinicBillingCodeManager
from ..clinics.models import CareTakingFacility, UniversalBillingCode, Clinic, ClinicMember, Company
from ..core.models import Auditable
from ..patients.models import Patient, InsuranceCompany

logger = logging.getLogger(__name__)


class ClinicBillingCode(Auditable, TimeStampedModel):
    display_order = models.IntegerField(_('Display order'), default=1000)
    universal_code = models.ForeignKey(UniversalBillingCode, verbose_name=_('universal billing code'),
                                       on_delete=models.PROTECT, related_name='clinic_billing_codes')
    clinic = models.ForeignKey(Clinic, verbose_name=_('clinic'), on_delete=models.PROTECT,
                               related_name='clinic_billing_codes')
    price = models.DecimalField(_('price'), max_digits=8, decimal_places=2)
    can_have_discount = models.BooleanField(_('can have discount'), default=True)
    activation_date = models.DateField(_('Activation Date'))
    expiration_date = models.DateField(_('Expiration date'), null=True, blank=True)

    objects = ClinicBillingCodeManager()

    class Meta:
        ordering = ('display_order', 'universal_code__description')
        verbose_name = _('clinic billing code')
        verbose_name_plural = _('clinic billing codes')

    def clean_fields(self, exclude=None):
        if self.expiration_date is not None and self.expiration_date < self.activation_date:
            raise ValidationError({'expiration_date': _('Expiration date cannot be before activation date')})

    def validate_unique(self, *args, **kwargs):
        super(ClinicBillingCode, self).validate_unique(*args, **kwargs)
        self._check_for_open()
        self._check_for_date_overlap()

    def _check_for_date_overlap(self):
        ol_filter = Q(
            expiration_date__gte=self.activation_date,
            universal_code=self.universal_code,
            clinic=self.clinic
        )
        if self.expiration_date is not None:
            ol_filter.add(Q(activation_date__lte=self.expiration_date, ), Q.AND)

        qs = self.__class__._default_manager.filter(ol_filter)

        if not self._state.adding and self.pk is not None:
            qs = qs.exclude(pk=self.pk)
        if qs.exists():
            raise ValidationError({
                NON_FIELD_ERRORS: ['overlapping date range', ],
            })

    def _check_for_open(self):
        # Issue
        qs = self.__class__._default_manager.filter(
            expiration_date__isnull=True,
            universal_code=self.universal_code,
            clinic=self.clinic
        )
        # TODO Verify if there is a better way to exclude
        if not self._state.adding and self.pk is not None:
            qs = qs.exclude(pk=self.pk)
            # logger.debug(f'Excluding {self.pk}')

        if qs.exists():
            count = qs.count()
            pks_list = list()
            # for obj in qs.all():
            #     pks_list.append(f'{obj.pk}')
            #     logger.debug(f'Pk:{obj.pk} code:{obj.universal_code.code} clinic:{obj.clinic}')
            msg = 'There is {} existing billing code(s) in this clinic ' \
                  'that are open (expiration date is null). Pks {}'.format(count, ','.join(pks_list))
            raise ValidationError({
                NON_FIELD_ERRORS: [msg, ],
            })

    def __str__(self):
        now = timezone.now().date()
        if self.expiration_date is not None and self.expiration_date > now:
            return _('{} - Standard Charge {} (expired)').format(self.universal_code, self.price)
        else:
            return _('{} - Standard Charge {}').format(self.universal_code, self.price)


class ChargeType(models.Model):
    """
    Charge types are used by accountants to classify what taxation is used fot the billing items.
    Current values are Emergency, Hospitalization, External Consultation and Home Visit.
    """
    code = models.CharField(_('code'), max_length=10, unique=True)
    name = models.CharField(_('name'), max_length=100)
    requires_hospital_info = models.BooleanField(_('requires hospital info'), default=False)

    def __str__(self):
        return self.name

    class Meta:
        ordering = ('name',)
        verbose_name = _('charge type')
        verbose_name_plural = _('charge types')


class ClinicChargeType(Auditable, TimeStampedModel):
    display_order = models.IntegerField(_('Display order'), default=1000)
    charge_type = models.ForeignKey(ChargeType, verbose_name=_('charge type'), on_delete=models.PROTECT)
    clinic = models.ForeignKey(Clinic, verbose_name=_('clinic'), on_delete=models.PROTECT)

    class Meta:
        ordering = ('display_order', 'charge_type__name')
        verbose_name = _('clinic charge type')
        verbose_name_plural = _('clinic charge types')


class Invoice(TimeStampedModel):
    BILL_TO_PATIENT = 'PATIENT'
    BILL_TO_INSURANCE = 'INSURANCE'
    BILL_TO_HOSPITAL = 'HOSPITAL'
    BILL_TO_OTHER = 'OTHER'
    BILL_TO_CHOICES = (
        (BILL_TO_PATIENT, _('Invoice to patient')),
        (BILL_TO_INSURANCE, _('Invoice to insurance company')),
        (BILL_TO_HOSPITAL, _('Invoice to Hospital')),
        (BILL_TO_OTHER, _('Invoice to Other')),
    )
    UNPAID_STATUS = 'UNPAID'
    PARTIAL_STATUS = 'PARTIAL'
    PAID_STATUS = 'PAID'
    OVER_PAID_STATUS = 'OVER_PAID'

    PAYMENT_STATUS_CHOICES = (
        (UNPAID_STATUS, _('Unpaid')),
        (PARTIAL_STATUS, _('Partially paid')),
        (PAID_STATUS, _('Paid')),
        (OVER_PAID_STATUS, _('Over paid')),
    )
    date = models.DateField(_('date'))
    bill_to = models.CharField(_('bill to'), max_length=10, choices=BILL_TO_CHOICES, default=BILL_TO_PATIENT)
    clinic = models.ForeignKey(Clinic, verbose_name=_('clinic'), on_delete=models.PROTECT,
                               related_name='invoices')
    company = models.ForeignKey(Company, verbose_name=_('Company'), related_name='invoices', null=True, blank=True,
                                on_delete=models.SET_NULL)
    patient = models.ForeignKey(Patient, verbose_name=_('patient'), related_name='invoices',
                                on_delete=models.PROTECT,
                                help_text='Patient. If no hospital or insurance company is'
                                          ' declared the bill is for the patient')
    hospital = models.ForeignKey(CareTakingFacility, verbose_name=_('hospital'),
                                 null=True, related_name='invoices', blank=True,
                                 on_delete=models.SET_NULL, help_text=_('Hospital to bill'))
    ticket = models.CharField(_('ticket'), max_length=15, blank=True, null=True)
    insurance_company = models.ForeignKey(InsuranceCompany, verbose_name=_('insurance company'),
                                          null=True, blank=True, related_name='invoices',
                                          on_delete=models.SET_NULL, help_text='Insurance Company to bill')
    insurance_policy = models.CharField(_('Insurance policy'), max_length=30, null=True, blank=True)
    discount = models.DecimalField(_('discount'), max_digits=13, decimal_places=12,
                                   help_text=_('Percentage of discount in decimal value.'),
                                   default=Decimal('0.00'))
    discountable_subtotal = models.DecimalField(_('discountable subtotal'), max_digits=8, decimal_places=2,
                                                default=Decimal('0.00'),
                                                help_text=_('Sum of all Billing itemas that are subject to discount'))
    non_discountable_subtotal = models.DecimalField(_('non discountable subtotal'), max_digits=8, decimal_places=2,
                                                    default=Decimal('0.00'),
                                                    help_text=_(
                                                        'Sum of all Billing itemas that are not subject to discount'))
    total = models.DecimalField(_('total'), max_digits=8, decimal_places=2, default=Decimal('0.00'))
    payment_status = models.CharField(_('payment status'), max_length=9, choices=PAYMENT_STATUS_CHOICES,
                                      default=UNPAID_STATUS)
    commission_paid = models.BooleanField(_('Is commission paid?'), default=False)
    commission_tickets = models.CharField(_('Commission tickets'), max_length=60, null=True, blank=True)

    objects = InvoiceManager()

    class Meta:
        ordering = ('-date', '-created')
        verbose_name = _('invoice')
        verbose_name_plural = _('invoices')

    def __str__(self):
        return _('Invoice {} for {} - {}').format(self.id, self.patient, self.total)

    def calculate(self):

        cents = Decimal('0.01')
        use_old = True
        if use_old:
            d_subtotal = Decimal('0.00')
            nd_subtotal = Decimal('0.00')
            for billing_item in self.billing_items.all():
                if billing_item.can_have_discount:
                    d_subtotal += billing_item.charge
                else:
                    nd_subtotal += billing_item.charge
            discount = (self.discount * d_subtotal).quantize(cents, ROUND_HALF_UP)
            total = d_subtotal + nd_subtotal - discount
        else:

            annotated = Invoice.objects.get_with_total(self.id)
            total = annotated.grand_total.quantize(cents, ROUND_HALF_UP)
            nd_subtotal = annotated.non_discountable_sum
            d_subtotal = annotated.discountable_sum

        self.total = total
        self.non_discountable_subtotal = nd_subtotal
        self.discountable_subtotal = d_subtotal
        # if self.amount_paid() == Decimal('0.00'):
        #     self.payment_status = self.PAID_STATUS
        self.set_payment_status()

    def amount_paid(self):
        amount_paid = self.invoice_payments.only('amount').all().aggregate(Sum('amount'))
        paid = Decimal(0.0) if amount_paid['amount__sum'] is None else amount_paid['amount__sum'];
        return paid

    def balance(self):
        paid = self.amount_paid()
        return self.total - paid

    def set_payment_status(self, commit=False):
        balance = self.balance()
        if balance == Decimal(0.0):
            self.payment_status = self.PAID_STATUS
        elif balance == self.total:
            self.payment_status = self.UNPAID_STATUS
        elif balance > Decimal(0.0):
            self.payment_status = self.PARTIAL_STATUS
        else:
            self.payment_status = self.OVER_PAID_STATUS

        if commit:
            self.save()

    def remove_encounter(self, encounter):
        if encounter.invoice == self:
            if self.payment_status == self.PAID_STATUS:
                msg = _('Invoice is paid. You cannot remove encounters from it.')
                raise FinanceException(msg)
            encounter.invoice = None
            encounter.save()
            self.calculate()
            self.save()
        else:
            msg = _('Encounter cannot be removed because it is not part of the bill.')
            raise FinanceException(msg)

    def pay(self, receipt_number, payment_type, user, **kwargs):
        if self.payment_status != self.PAID_STATUS:
            payment_data = dict()
            payment_data['date'] = kwargs.get('date', timezone.now().today())
            payment_data['receipt_number'] = receipt_number
            payment_data['clinic'] = self.clinic
            payment_data['amount'] = kwargs.get('amount', self.total)
            payment_data['payment_type'] = payment_type
            payment_data['bank'] = kwargs.get('bank', None)
            payment_data['transaction_num'] = kwargs.get('transaction_num', None)
            payment_data['created_by'] = user
            payment_data['modified_by'] = user
            with transaction.atomic():
                try:
                    payment = Payment.objects.create(**payment_data)

                    invoice_payment_data = dict()
                    invoice_payment_data['invoice'] = self
                    invoice_payment_data['payment'] = payment
                    invoice_payment_data['amount'] = payment.amount
                    invoice_payment_data['created_by'] = user
                    invoice_payment_data['modified_by'] = user

                    InvoicePayment.objects.create(**invoice_payment_data)

                    self.calculate()
                    self.save()
                    return payment
                except IntegrityError as ie:
                    raise FinanceException(str(ie))
        else:
            msg = _('Cannot pay a paid invoice.')
            raise FinanceException(msg)


class BillingItem(TimeStampedModel):
    date = models.DateTimeField(_('date'))
    patient = models.ForeignKey(Patient, verbose_name=_('patient'), related_name='billing_items',
                                on_delete=models.PROTECT)
    hospital = models.ForeignKey(CareTakingFacility, verbose_name=_('hospital'),
                                 null=True, blank=True,
                                 on_delete=models.SET_NULL)
    ticket = models.CharField(_('ticket'), max_length=15, blank=True, null=True)
    billing_code = models.ForeignKey(ClinicBillingCode, verbose_name=_('billing code'),
                                     related_name='billing_items', on_delete=models.PROTECT)
    charge = models.DecimalField(_('charge'), max_digits=8, decimal_places=2)
    description = models.CharField(_('description'), max_length=120, blank=True,
                                   help_text='Description for when the billing code is OTHER')
    charge_type = models.ForeignKey(ChargeType, on_delete=models.PROTECT,
                                    verbose_name=_('charge type'), related_name='billing_items')
    physician = models.ForeignKey(ClinicMember, on_delete=models.PROTECT, verbose_name=_('physician'))
    clinic = models.ForeignKey(Clinic, verbose_name=_('clinic'), on_delete=models.PROTECT)
    invoice = models.ForeignKey(Invoice, verbose_name=_('invoice'), null=True, blank=True, on_delete=models.SET_NULL,
                                related_name='billing_items')
    created_by = models.ForeignKey(settings.AUTH_USER_MODEL, on_delete=models.SET_NULL, null=True)
    can_have_discount = models.BooleanField(_('can have discount'), default=True)

    objects = EncounterManager()

    class Meta:
        ordering = ('-date', '-created')
        verbose_name = _('encounter')
        verbose_name_plural = _('encounters')

    def __str__(self):
        return '{} {} {}'.format(self.billing_code.universal_code.code, self.billing_code.universal_code.description,
                                 self.charge)


class Payment(Auditable, TimeStampedModel):
    CASH_PAYMENT = 'CASH'
    CREDIT_CARD_PAYMENT = 'CC'
    DEBIT_CARD_PAYMENT = 'DEBIT'
    ACH_PAYMENT = 'ACH'
    CHECK_PAYMENT = 'CHECK'

    PAYMENT_TYPE_CHOICES = (
        (CASH_PAYMENT, _('Cash')),
        (CREDIT_CARD_PAYMENT, _('Credit Card')),
        (DEBIT_CARD_PAYMENT, _('Debit card')),
        (ACH_PAYMENT, _('ACH')),
        (CHECK_PAYMENT, _('Check')),
    )

    date = models.DateField(_('date'))
    receipt_number = models.CharField(_('receipt number'), max_length=15)
    clinic = models.ForeignKey(Clinic, verbose_name=_('clinic'), related_name='payments', on_delete=models.PROTECT)
    amount = models.DecimalField(_('amount'), max_digits=12, decimal_places=2)
    payment_type = models.CharField(_('payment type'), max_length=5, choices=PAYMENT_TYPE_CHOICES,
                                    default=CREDIT_CARD_PAYMENT)
    bank = models.CharField(_('bank'), max_length=30, null=True, blank=True)
    transaction_num = models.CharField(_('transaction num'), max_length=30, null=True, blank=True)

    class Meta:
        verbose_name = _('payment')
        verbose_name_plural = _('payments')

    def __str__(self):
        return '{} {}'.format(self.receipt_number, self.amount)


class InvoicePayment(Auditable, TimeStampedModel):
    invoice = models.ForeignKey(Invoice, verbose_name=_('Invoice'), related_name='invoice_payments',
                                on_delete=models.CASCADE)
    payment = models.ForeignKey(Payment, verbose_name=_('Payment'), related_name='invoice_payments',
                                on_delete=models.CASCADE)
    amount = models.DecimalField(_('Amount'), max_digits=12, decimal_places=2)

    class Meta:
        verbose_name = _('invoice payment')
        verbose_name_plural = _('invoice payments')



"""Console script for django_scaffolding_tools."""
import logging
import os
import sys
from pathlib import Path
from platform import python_version

import click

from django_scaffolding_tools.django.cli import models_to_json
from django_scaffolding_tools.enums import CommandType
from django_scaffolding_tools.writers import write_serializer_from_file
from . import __version__ as current_version

logger = logging.getLogger(__name__)


@click.command()
@click.argument('command')
@click.option('--source-file')
@click.option('--output-folder')
def main2(command, source_file, output_folder):
    """Console script for django_scaffolding_tools."""
    click.echo(f"See click documentation {os.getcwd()}")
    if command == CommandType.JSON_TO_SERIALIZER:
        source_file_path = Path(source_file)
        output_folder = Path(output_folder)
        if output_folder.exists():
            target_file = output_folder / '__serializers.py'
            click.echo(f'JSON to serializer from {source_file_path} to {target_file}')
            try:
                write_serializer_from_file(source_file_path, target_file)
            except Exception as e:
                error_message = f'Type: {e.__class__.__name__} Error: {e}'
                print(error_message)
                return 200
            return 0
        else:
            print(f'NO output folder {output_folder}')
            raise Exception('No output folder')


@click.group()
@click.version_option(version=current_version)
def main():
    pass


@click.command()
def about():
    banner_char = '-'
    app_name = 'Django Scaffolding Tools'
    length = len(app_name) + 4
    click.echo(banner_char * length)
    click.echo(f'{banner_char} {app_name} {banner_char}')
    click.echo(banner_char * length)
    click.echo(f'Operating System: {sys.platform}')
    click.echo(f'Python : {python_version()}')
    logger.debug('Ran about command.')


main.add_command(about)
main.add_command(models_to_json)

if __name__ == "__main__":
    sys.exit(main())  # pragma: no cover



# import the required libraries
import json
import pickle
import os.path
from mimetypes import MimeTypes
from pathlib import Path

from googleapiclient.discovery import build
from google_auth_oauthlib.flow import InstalledAppFlow
from google.auth.transport.requests import Request
from googleapiclient.http import MediaFileUpload

from django_scaffolding_tools._experimental.backup_envs.exceptions import UploadError


class GDrive:
    SCOPES = ['https://www.googleapis.com/auth/drive']

    def __init__(self, secrets_file: Path):
        self.secrets_file = secrets_file
        token_file = secrets_file.parent / 'token.pickle'
        creds = self.get_g_drive_credentials(token_file)
        self.service = build('drive', 'v3', credentials=creds)

    def get_g_drive_credentials(self, token_file):
        creds = None
        # The file token.pickle stores the
        # user's access and refresh tokens. It is
        # created automatically when the authorization
        # flow completes for the first time.
        # Check if file token.pickle exists
        if token_file.exists():
            # Read the token from the file and
            # store it in the variable creds
            with open(token_file, 'rb') as token:
                creds = pickle.load(token)
        # If no valid credentials are available,
        # request the user to log in.
        if not creds or not creds.valid:

            # If token is expired, it will be refreshed,
            # else, we will request a new one.
            if creds and creds.expired and creds.refresh_token:
                creds.refresh(Request())
            else:
                flow = InstalledAppFlow.from_client_secrets_file(str(self.secrets_file), self.SCOPES)
                creds = flow.run_local_server(port=0)

            # Save the access token in token.pickle
            # file for future usage
            with open(token_file, 'wb') as token:
                pickle.dump(creds, token)
        return creds

    def upload(self, file_to_upload: Path, folder_id: str):
        filename = file_to_upload.name
        mime_type = 'application/octet-stream'
        body = {'name': filename, 'parents': [folder_id], 'mimeType': mime_type}
        try:
            media_body = MediaFileUpload(file_to_upload, mimetype=mime_type, chunksize=10485760, resumable=True)
            request = self.service.files().create(body=body, media_body=media_body)  # Modified
            result = request.execute()
            return result
        except Exception as e:
            error_message = f'Upload error. Type {e.__class__.__name__} error {e}'
            raise UploadError(error_message)

    # Define the SCOPES. If modifying it,
    # delete the token.pickle file.


SCOPES = ['https://www.googleapis.com/auth/drive']


# Create a function get_file_list with
# parameter N which is the length of
# the list of files.


def get_file_list(N):
    g_drive_folder = Path(__file__).parent.parent.parent.parent / '.envs' / 'google_drive'
    secrets_file = g_drive_folder / 'client_secrets.json'
    token_file = g_drive_folder / 'token.pickle'
    # Variable creds will store the user access token.
    # If no valid token found, we will create one.
    creds = get_g_drive_credentials(secrets_file, token_file)

    # Connect to the API service
    service = build('drive', 'v3', credentials=creds)

    # request a list of first N files or
    # folders with name and id from the API.
    resource = service.files()
    result = resource.list(pageSize=N, fields="files(id, name)",
                           q="mimeType = 'application/vnd.google-apps.folder' and name = 'Envs'", ).execute()

    # return the result dictionary containing
    # the information about the files
    return result


def get_g_drive_credentials(secrets_file, token_file):
    creds = None
    # The file token.pickle stores the
    # user's access and refresh tokens. It is
    # created automatically when the authorization
    # flow completes for the first time.
    # Check if file token.pickle exists
    if token_file.exists():
        # Read the token from the file and
        # store it in the variable creds
        with open(token_file, 'rb') as token:
            creds = pickle.load(token)
    # If no valid credentials are available,
    # request the user to log in.
    if not creds or not creds.valid:

        # If token is expired, it will be refreshed,
        # else, we will request a new one.
        if creds and creds.expired and creds.refresh_token:
            creds.refresh(Request())
        else:
            flow = InstalledAppFlow.from_client_secrets_file(str(secrets_file), SCOPES)
            creds = flow.run_local_server(port=0)

        # Save the access token in token.pickle
        # file for future usage
        with open(token_file, 'wb') as token:
            pickle.dump(creds, token)
    return creds


def old_way():
    # Get list of first 5 files or
    # folders from our Google Drive Storage
    result_dict = get_file_list(150)

    # Extract the list from the dictionary
    file_list = result_dict.get('files')

    # Print every file's name
    print(f'Results {len(file_list)}')
    for file in file_list:
        if file['name'] == 'Envs':
            print(file['name'], file['id'])
        else:
            print('*', end='', flush=False)


def main():
    g_drive_folder = Path(__file__).parent.parent.parent.parent / '.envs' / 'google_drive'
    secrets_file = g_drive_folder / 'client_secrets.json'

    folder_ids_file = g_drive_folder / 'google_drive_folders_id.json'
    with open(folder_ids_file, 'r') as json_file:
        folder_ids = json.load(json_file)
    folder_id = folder_ids['Envs']

    gdrive = GDrive(secrets_file)
    test_file = Path(
        '/home/luiscberrocal/PycharmProjects/django_scaffolding_tools/.envs/google_drive/google_drive_folders_id.json')
    r = gdrive.upload(test_file, folder_id)
    assert r == ''

if __name__ == '__main__':
    main()



import ast
import csv
import re
from operator import itemgetter
from pathlib import Path
from typing import Dict, Any, List, Callable, Tuple

import humps
from ast2json import ast2json

from django_scaffolding_tools.enums import NativeDataType, PatternType


def to_snake_case(name: str) -> str:
    name = re.sub('(.)([A-Z][a-z]+)', r'\1_\2', name)
    name = re.sub('__([A-Z])', r'_\1', name)
    name = re.sub('([a-z0-9])([A-Z])', r'\1_\2', name)
    return name.lower()


def transform_dict_to_model_list(data: Dict[str, Any]) -> List[Dict[str, Any]]:
    model_list = list()
    for key, model in data.items():
        model_list.append(model)
        for att in model['attributes']:
            if not att['native']:
                model_list += transform_dict_to_model_list(att['value'])
    return sorted(model_list, key=itemgetter('level'), reverse=True)


def parse_var_name(var_name: str) -> Tuple[str, bool]:
    """Transforms a variable to snake case from pascal and camel case."""
    if humps.is_snakecase(var_name):
        new_var_name = var_name
    elif humps.is_camelcase(var_name):
        new_var_name = humps.decamelize(var_name)
    elif humps.is_pascalcase(var_name):
        new_var_name = humps.depascalize(var_name)
    else:
        raise Exception('Unsupported casing.')
    return new_var_name, new_var_name != var_name


def parse_dict(data: Dict[str, Any], model_name: str = 'Model', level: int = 0,
               value_item_level: int = 0) -> Dict[str, Any]:
    """Parses a dictionary containing data to create a basic Model dictionary"""
    parsed_dict: dict[str, dict[Any, Any] | list[Any]] = dict()
    key_name = to_snake_case(model_name)
    parsed_dict[key_name] = dict()
    parsed_dict[key_name]['name'] = model_name
    parsed_dict[key_name]['level'] = level
    parsed_dict[key_name]['attributes'] = list()
    for key, item in data.items():
        variable_name, add_alias = parse_var_name(key)

        item_data = {'name': variable_name, 'value': item, 'supported': False, 'native': True, 'many': False}
        if add_alias:
            item_data['alias'] = key
        data_type = item.__class__.__name__
        if data_type in NativeDataType.to_list():
            item_data['data_type'] = data_type
            item_data['supported'] = True
            if data_type == NativeDataType.STRING.value:
                item_data['length'] = len(item)
        elif isinstance(item, dict):
            pascalized_model_name = humps.pascalize(key)
            item_data['data_type'] = pascalized_model_name
            item_data['value'] = parse_dict(item, model_name=pascalized_model_name, level=level + 1)
            item_data['supported'] = True
            item_data['native'] = False
        elif isinstance(item, list):
            value_item_level += 1
            for value_item in item:
                model_name = f'ValueItem{value_item_level}'
                item_data['data_type'] = model_name
                item_data['value'] = parse_dict(value_item, model_name=model_name, level=level + 1,
                                                value_item_level=value_item_level + 1)
                item_data['supported'] = True
                item_data['native'] = False
                item_data['many'] = True
        else:
            item_data['data_type'] = data_type

        parsed_dict[key_name]['attributes'].append(item_data)
    return parsed_dict


def parse_for_patterns(model_list: List[Dict[str, Any]],
                       pattern_functions: List[Callable[[str], PatternType]]) -> List[Dict[str, Any]]:
    for model in model_list:
        for attribute in model['attributes']:
            if attribute['data_type'] == NativeDataType.STRING:
                for pattern_function in pattern_functions:
                    pattern = pattern_function(attribute['value'])
                    if pattern is not None:
                        attribute['pattern_type'] = pattern
                        break
    return model_list


SERIALIZER_FIELDS = {
    NativeDataType.STRING: {'field': 'CharField'},
    NativeDataType.INTEGER: {'field': 'IntegerField'},
    NativeDataType.FLOAT: {'field': 'FloatField'},
    NativeDataType.DATE: {'field': 'DateField'},
    NativeDataType.DATETIME: {'field': 'DatetimeField'},
    PatternType.DATE: {'field': 'DateField'},
    PatternType.DATETIME: {'field': 'DatetimeField'},
    PatternType.URL: {'field': 'UrlField'},
    PatternType.EMAIL: {'field': 'EmailField'},
}


def parse_file_for_ast_classes(filename: Path) -> Dict[str, Any]:
    with open(filename, 'r') as py_file:
        content = py_file.read()
        node = ast.parse(content)
    node_dict = ast2json(node)
    return node_dict


def parse_file_for_enum(csv_file: Path, delimiter:str=',') -> List[Dict[str, Any]]:
    """Parses as csv file with value, name and description"""
    enumerations = list()
    with open(csv_file, 'r') as file:
        reader = csv.DictReader(file, delimiter=delimiter)
        for row in reader:
            row['value'] = row['value'].strip()
            row['name'] = row['name'].strip().upper().replace(' ','_').replace('-', '_')
            row['description'] = row['description'].strip()
            # print(row)
            enumerations.append(row)
    return enumerations





from click.testing import CliRunner

from django_scaffolding_tools.django import cli


def test_model_2_serializer(output_folder, fixtures_folder):
    out_file = output_folder / 'serializers.py'
    out_file.unlink(missing_ok=True)

    runner = CliRunner()
    commands = ['model_2_serializer', '--folder', fixtures_folder, '--output-folder', output_folder]
    help_result = runner.invoke(cli.main, commands)
    print(help_result)

    assert help_result.exit_code == 0



import subprocess
from typing import List, Tuple


def run_commands(commands: List[str], encoding: str = 'utf-8') -> Tuple[List[str], List[str]]:
    """
    :param commands: <list> The command and paraemters to run
    :param encoding: <str> Encoding for the shell
    :return: <tuple> Containing 2 lists. First one with results and the Second one with errors if any.
    """
    result = subprocess.run(commands,
                            stdout=subprocess.PIPE,
                            stderr=subprocess.PIPE,
                            stdin=subprocess.PIPE)
    result_lines = result.stdout.decode(encoding).split('\n')[:-1]
    error_lines = result.stderr.decode(encoding).split('\n')[:-1]
    return result_lines, error_lines



import json

from django_scaffolding_tools.django.handlers import IntegerFieldHandler, DateFieldHandler, DateTimeFieldHandler, \
    CharFieldHandler, ForeignKeyFieldHandler, DecimalFieldHandler, BooleanFieldHandler, DateTimeCharFieldHandler


def test_handlers():
    field_data = {
        "name": "financial_product_id",
        "keywords": [
            {
                "name": "max_length",
                "value_type_TMP": "Constant",
                "value": 64
            }
        ],
        "arguments": [],
        "data_type": "CharField"
    }

    int_handler = IntegerFieldHandler()
    char_handler = CharFieldHandler()

    int_handler.set_next(char_handler)

    result = int_handler.handle(field_data)
    expected = 'LazyAttribute(lambda x: FuzzyText(length=64, chars=string.digits).fuzz())'
    assert result['factory_field'] == expected


def test_date_time_char_handler():
    field_list = [{
        "name": "payment_date",
        "keywords": [
            {
                "name": "max_length",
                "value_type_TMP": "Constant",
                "value": 15
            }
        ],
        "arguments": [],
        "data_type": "CharField"
    },
        {
            "name": "payment_id",
            "keywords": [
                {
                    "name": "max_length",
                    "value_type_TMP": "Constant",
                    "value": 16
                }
            ],
            "arguments": [],
            "data_type": "CharField"
        }
    ]

    handlers = [
        DateTimeCharFieldHandler(),
        CharFieldHandler()
    ]

    for i in range(len(handlers)):
        if i < len(handlers) - 1:
            handlers[i].set_next(handlers[i + 1])

    handler = handlers[0]
    expected_values = [
        'LazyAttribute(lambda x: faker.date_time_between(start_date="-1y", '
        'end_date="now", tzinfo=timezone(settings.TIME_ZONE)).strftime(%Y-%m-%dT%H:%M:%S%z))',
        'LazyAttribute(lambda x: FuzzyText(length=16, chars=string.digits).fuzz())'
    ]
    for i, field_data in enumerate(field_list):
        result = handler.handle(field_data)
        assert result['factory_field'] == expected_values[i]  # , f'assertion error with {field_data["name"]}'


def test_all(fixtures_folder, output_folder):
    file = fixtures_folder / 'model_data_models_payements.py.json'
    file = output_folder / 'django' / 'model_data_models.py.json'

    with open(file, 'r') as json_file:
        class_data = json.load(json_file)
    handlers = [
        IntegerFieldHandler(),
        DateTimeCharFieldHandler(),
        CharFieldHandler(),
        ForeignKeyFieldHandler(),
        DateFieldHandler(),
        DateTimeFieldHandler(),
        DecimalFieldHandler(),
        BooleanFieldHandler()
    ]

    for i in range(len(handlers)):
        if i < len(handlers) - 1:
            handlers[i].set_next(handlers[i + 1])
    print('----------------------------------------------------------')
    main_handler = handlers[0]
    for fp_data in class_data['classes']:
        # fp_data = class_data['classes'][3]
        print(f'class {fp_data["name"]}Factory(DjangoModelFactory):')
        print(f'\tclass Meta:')
        print(f'\t\tmodel = {fp_data["name"]}')
        print("")
        for att in fp_data['attributes']:
            result = main_handler.handle(att)
            if result is None:
                print(f'\t# {att["name"]} {att["data_type"]} NOT supported')
            else:
                print(f'\t{result["name"]} = {result["factory_field"]}')
        print("#" * 80)


def test_iteration():
    handlers = [
        IntegerFieldHandler(),
        DateTimeCharFieldHandler(),
        CharFieldHandler(),
        ForeignKeyFieldHandler(),
        DateFieldHandler(),
        DateTimeFieldHandler(),
        DecimalFieldHandler(),
        BooleanFieldHandler()
    ]

    for i in range(len(handlers)):
        if i < len(handlers) - 1:
            handlers[i].set_next(handlers[i + 1])

    main_handler = handlers[0]

    handler_list = list(main_handler)
    for i, handler in enumerate(handler_list):
        assert handler == handlers[i + 1]

    assert len(main_handler) == len(handlers) - 1



import csv
import json
from pathlib import Path
from typing import Any, Dict, List, Union

from jinja2 import PackageLoader, Environment

from django_scaffolding_tools.builders import build_serializer_data
from django_scaffolding_tools.exceptions import DSTException
from django_scaffolding_tools.parsers import parse_dict, transform_dict_to_model_list, parse_for_patterns
from django_scaffolding_tools.patterns import PATTERN_FUNCTIONS


class ReportWriter:

    def __init__(self):
        self.template_env = Environment(loader=PackageLoader('django_scaffolding_tools', 'templates'))

    def write(self, template_name: str, output_file: Path, **params):
        template = self.template_env.get_template(template_name)
        output = template.render(**params)
        with open(output_file, 'w') as html_file:
            html_file.write(output)


def write_serializer_from_file(source_file: Path, output_file: Path):
    with open(source_file, 'r') as json_file:
        data = json.load(json_file)

    writer = ReportWriter()
    parsed_dict = parse_dict(data)

    model_list = transform_dict_to_model_list(parsed_dict)
    model_list = parse_for_patterns(model_list, PATTERN_FUNCTIONS)
    model_list = build_serializer_data(model_list)

    writer.write('serializers.py.j2', output_file, model_list=model_list)


def get_keyword(att_keywords: List[Dict[str, Any]], keyword_name: str) -> Union[str, int]:
    for att_keyword in att_keywords:
        try:
            if att_keyword['name'] == keyword_name:
                return att_keyword['value']
        except TypeError as e:
            error_message = f'Error getting keyword {keyword_name}. Error: {e}'
            raise DSTException(error_message)


def write_django_model_csv(models_list: List[Dict[str, Any]], filename: Path):
    with open(filename, 'w') as csv_file:
        writer = csv.writer(csv_file)
        writer.writerow(['Class name', 'Attribute', 'Field type', 'Max length', 'Description'])
        for model in models_list:
            class_name = model['name']
            for att in model['attributes']:
                att_name = att['name']
                att_field_type = att['data_type']
                max_length = get_keyword(att['keywords'], 'max_length')
                description = get_keyword(att['keywords'], 'help_text')
                if description is not None:
                    description = description.replace(',', ' ')
                writer.writerow([class_name, att_name, att_field_type, max_length, description])


def write_django_model_excel(models_list: List[Dict[str, Any]], filename: Path):
    headers = {
        'class_name': {'title': 'Class name'},
        'attribute': {'title': 'Attribute name'},
        'field_type': {'title': 'Field type'},
        'max_value': {'title': 'Max value'},
        'description': {'title': 'Description'}
    }
    excel_data = list()
    for model in models_list:
        for att in model['attributes']:
            excel_dict = {'class_name': model['name'], 'attribute': att['name'],
                          'field_type': att['data_type']}
            max_value = get_keyword(att['keywords'], 'max_length')
            if max_value is not None:
                max_value = f'max length={max_value}'
            else:
                max_digits = get_keyword(att['keywords'], 'max_digits')
                decimal_places = get_keyword(att['keywords'], 'decimal_places')
                if max_digits is not None:
                    max_value = f'digits={max_digits}, decimal places={decimal_places}'
            excel_dict['max_value'] = max_value
            excel_dict['description'] = get_keyword(att['keywords'], 'help_text')
            excel_data.append(excel_dict)
    from django_scaffolding_tools.utils.core import quick_write
    quick_write(excel_data, filename.parent / '__excel.json')
    simple_write_to_excel(filename, headers, excel_data)


def simple_write_to_excel(filename: Path, headers: Dict[str, Any], lines: List[Dict[str, Any]]):
    """Writes to excel a List of dictionaries. The headers keys must match the keys of the values to write.
    The headers must contain a dictionary with the title key."""
    from openpyxl.workbook import Workbook
    wb = Workbook()
    sheet = wb.create_sheet()
    row = 1
    col = 1
    for key, header in headers.items():
        sheet.cell(row=row, column=col, value=header['title'])
        col += 1
    row += 1
    for line in lines:
        col = 1
        for key in headers.keys():
            value = line.get(key)
            if value is not None:
                sheet.cell(row=row, column=col, value=value)
            col += 1
        row += 1

    wb.save(filename)

def write_enums(enum_name:str, enumerations: List[Dict[str, Any]], module_file: Path) -> None:
    lines = list()
    line = f'class {enum_name}(str, Enum):\n'
    lines.append(line)
    for enumeration in enumerations:
        line = f'\t{enumeration["name"]} = \'{enumeration["value"]}\'\n'
        lines.append(line)
    with open(module_file, 'w') as m_file:
        m_file.writelines(lines)



import json
from datetime import datetime

import humps
import pytest

from django_scaffolding_tools.builders import build_serializer_template_data, build_serializer_data
from django_scaffolding_tools.parsers import parse_dict, transform_dict_to_model_list, parse_for_patterns, \
    parse_file_for_ast_classes, parse_var_name, parse_file_for_enum
from django_scaffolding_tools.patterns import PATTERN_FUNCTIONS
from django_scaffolding_tools.utils.core import quick_write
from django_scaffolding_tools.writers import ReportWriter


class TestParseDataDictionary:

    def test_parse(self, d_local_data_dict):
        parsed_dict = parse_dict(d_local_data_dict, model_name='Payment')

        assert parsed_dict['payment']['level'] == 0
        assert parsed_dict['payment']['attributes'][0]['name'] == 'id'
        assert parsed_dict['payment']['attributes'][0]['value'] == 'D-4-be8eda8c-5fe7-49dd-8058-4ddaac00611b'
        assert parsed_dict['payment']['attributes'][0]['supported']
        assert parsed_dict['payment']['attributes'][0]['native']
        assert not parsed_dict['payment']['attributes'][0]['many']
        assert parsed_dict['payment']['attributes'][0]['data_type'] == 'str'
        assert parsed_dict['payment']['attributes'][0]['length'] == 40
        assert parsed_dict['payment']['attributes'][1]['name'] == 'amount'
        assert parsed_dict['payment']['attributes'][1]['value'] == 72.0
        assert parsed_dict['payment']['attributes'][1]['supported']
        assert parsed_dict['payment']['attributes'][1]['native']
        assert not parsed_dict['payment']['attributes'][1]['many']
        assert parsed_dict['payment']['attributes'][1]['data_type'] == 'float'
        assert parsed_dict['payment']['attributes'][2]['name'] == 'status'
        assert parsed_dict['payment']['attributes'][2]['value'] == 'PAID'
        assert parsed_dict['payment']['attributes'][2]['supported']
        assert parsed_dict['payment']['attributes'][2]['native']
        assert not parsed_dict['payment']['attributes'][2]['many']
        assert parsed_dict['payment']['attributes'][2]['data_type'] == 'str'
        assert parsed_dict['payment']['attributes'][2]['length'] == 4
        assert parsed_dict['payment']['attributes'][3]['name'] == 'status_detail'
        assert parsed_dict['payment']['attributes'][3]['value'] == 'The payment was paid.'
        assert parsed_dict['payment']['attributes'][3]['supported']
        assert parsed_dict['payment']['attributes'][3]['native']
        assert not parsed_dict['payment']['attributes'][3]['many']
        assert parsed_dict['payment']['attributes'][3]['data_type'] == 'str'
        assert parsed_dict['payment']['attributes'][3]['length'] == 21
        assert parsed_dict['payment']['attributes'][4]['name'] == 'status_code'
        assert parsed_dict['payment']['attributes'][4]['value'] == '200'
        assert parsed_dict['payment']['attributes'][4]['supported']
        assert parsed_dict['payment']['attributes'][4]['native']
        assert not parsed_dict['payment']['attributes'][4]['many']
        assert parsed_dict['payment']['attributes'][4]['data_type'] == 'str'
        assert parsed_dict['payment']['attributes'][4]['length'] == 3
        assert parsed_dict['payment']['attributes'][5]['name'] == 'currency'
        assert parsed_dict['payment']['attributes'][5]['value'] == 'USD'
        assert parsed_dict['payment']['attributes'][5]['supported']
        assert parsed_dict['payment']['attributes'][5]['native']
        assert not parsed_dict['payment']['attributes'][5]['many']
        assert parsed_dict['payment']['attributes'][5]['data_type'] == 'str'
        assert parsed_dict['payment']['attributes'][5]['length'] == 3
        assert parsed_dict['payment']['attributes'][6]['name'] == 'country'
        assert parsed_dict['payment']['attributes'][6]['value'] == 'AR'
        assert parsed_dict['payment']['attributes'][6]['supported']
        assert parsed_dict['payment']['attributes'][6]['native']
        assert not parsed_dict['payment']['attributes'][6]['many']
        assert parsed_dict['payment']['attributes'][6]['data_type'] == 'str'
        assert parsed_dict['payment']['attributes'][6]['length'] == 2
        assert parsed_dict['payment']['attributes'][7]['name'] == 'payment_method_id'
        assert parsed_dict['payment']['attributes'][7]['value'] == 'RP'
        assert parsed_dict['payment']['attributes'][7]['supported']
        assert parsed_dict['payment']['attributes'][7]['native']
        assert not parsed_dict['payment']['attributes'][7]['many']
        assert parsed_dict['payment']['attributes'][7]['data_type'] == 'str'
        assert parsed_dict['payment']['attributes'][7]['length'] == 2
        assert parsed_dict['payment']['attributes'][8]['name'] == 'payment_method_type'
        assert parsed_dict['payment']['attributes'][8]['value'] == 'TICKET'
        assert parsed_dict['payment']['attributes'][8]['supported']
        assert parsed_dict['payment']['attributes'][8]['native']
        assert not parsed_dict['payment']['attributes'][8]['many']
        assert parsed_dict['payment']['attributes'][8]['data_type'] == 'str'
        assert parsed_dict['payment']['attributes'][8]['length'] == 6
        assert parsed_dict['payment']['attributes'][9]['name'] == 'payment_method_flow'
        assert parsed_dict['payment']['attributes'][9]['value'] == 'REDIRECT'
        assert parsed_dict['payment']['attributes'][9]['supported']
        assert parsed_dict['payment']['attributes'][9]['native']
        assert not parsed_dict['payment']['attributes'][9]['many']
        assert parsed_dict['payment']['attributes'][9]['data_type'] == 'str'
        assert parsed_dict['payment']['attributes'][9]['length'] == 8
        assert parsed_dict['payment']['attributes'][10]['name'] == 'payer'
        assert parsed_dict['payment']['attributes'][10]['value']['payer']['name'] == 'Payer'
        assert parsed_dict['payment']['attributes'][10]['value']['payer']['level'] == 1
        assert parsed_dict['payment']['attributes'][10]['value']['payer']['attributes'][0]['name'] == 'name'
        assert parsed_dict['payment']['attributes'][10]['value']['payer']['attributes'][0]['value'] == 'Nino Deicas'
        assert parsed_dict['payment']['attributes'][10]['value']['payer']['attributes'][0]['supported']
        assert parsed_dict['payment']['attributes'][10]['value']['payer']['attributes'][0]['native']
        assert not parsed_dict['payment']['attributes'][10]['value']['payer']['attributes'][0]['many']
        assert parsed_dict['payment']['attributes'][10]['value']['payer']['attributes'][0]['data_type'] == 'str'
        assert parsed_dict['payment']['attributes'][10]['value']['payer']['attributes'][0]['length'] == 11
        assert parsed_dict['payment']['attributes'][10]['value']['payer']['attributes'][1]['name'] == 'user_reference'
        assert parsed_dict['payment']['attributes'][10]['value']['payer']['attributes'][1][
                   'value'] == 'US-jmh3gb4kj5h34'
        assert parsed_dict['payment']['attributes'][10]['value']['payer']['attributes'][1]['supported']
        assert parsed_dict['payment']['attributes'][10]['value']['payer']['attributes'][1]['native']
        assert not parsed_dict['payment']['attributes'][10]['value']['payer']['attributes'][1]['many']
        assert parsed_dict['payment']['attributes'][10]['value']['payer']['attributes'][1]['data_type'] == 'str'
        assert parsed_dict['payment']['attributes'][10]['value']['payer']['attributes'][1]['length'] == 16
        assert parsed_dict['payment']['attributes'][10]['value']['payer']['attributes'][2]['name'] == 'email'
        assert parsed_dict['payment']['attributes'][10]['value']['payer']['attributes'][2]['value'] == 'buyer@gmail.com'
        assert parsed_dict['payment']['attributes'][10]['value']['payer']['attributes'][2]['supported']
        assert parsed_dict['payment']['attributes'][10]['value']['payer']['attributes'][2]['native']
        assert not parsed_dict['payment']['attributes'][10]['value']['payer']['attributes'][2]['many']
        assert parsed_dict['payment']['attributes'][10]['value']['payer']['attributes'][2]['data_type'] == 'str'
        assert parsed_dict['payment']['attributes'][10]['value']['payer']['attributes'][2]['length'] == 15
        assert parsed_dict['payment']['attributes'][10]['value']['payer']['attributes'][3]['name'] == 'address'
        assert parsed_dict['payment']['attributes'][10]['value']['payer']['attributes'][3]['value']['address'][
                   'name'] == 'Address'
        assert parsed_dict['payment']['attributes'][10]['value']['payer']['attributes'][3]['value']['address'][
                   'level'] == 2
        assert \
            parsed_dict['payment']['attributes'][10]['value']['payer']['attributes'][3]['value']['address'][
                'attributes'][0][
                'name'] == 'street'
        assert \
            parsed_dict['payment']['attributes'][10]['value']['payer']['attributes'][3]['value']['address'][
                'attributes'][0][
                'value'] == '123th street'
        assert \
            parsed_dict['payment']['attributes'][10]['value']['payer']['attributes'][3]['value']['address'][
                'attributes'][0][
                'supported']
        assert \
            parsed_dict['payment']['attributes'][10]['value']['payer']['attributes'][3]['value']['address'][
                'attributes'][0][
                'native']
        assert not \
            parsed_dict['payment']['attributes'][10]['value']['payer']['attributes'][3]['value']['address'][
                'attributes'][0][
                'many']
        assert \
            parsed_dict['payment']['attributes'][10]['value']['payer']['attributes'][3]['value']['address'][
                'attributes'][0][
                'data_type'] == 'str'
        assert \
            parsed_dict['payment']['attributes'][10]['value']['payer']['attributes'][3]['value']['address'][
                'attributes'][0][
                'length'] == 12
        assert \
            parsed_dict['payment']['attributes'][10]['value']['payer']['attributes'][3]['value']['address'][
                'attributes'][1][
                'name'] == 'state'
        assert \
            parsed_dict['payment']['attributes'][10]['value']['payer']['attributes'][3]['value']['address'][
                'attributes'][1][
                'value'] == 'FL'
        assert \
            parsed_dict['payment']['attributes'][10]['value']['payer']['attributes'][3]['value']['address'][
                'attributes'][1][
                'supported']
        assert \
            parsed_dict['payment']['attributes'][10]['value']['payer']['attributes'][3]['value']['address'][
                'attributes'][1][
                'native']
        assert not \
            parsed_dict['payment']['attributes'][10]['value']['payer']['attributes'][3]['value']['address'][
                'attributes'][1][
                'many']
        assert \
            parsed_dict['payment']['attributes'][10]['value']['payer']['attributes'][3]['value']['address'][
                'attributes'][1][
                'data_type'] == 'str'
        assert \
            parsed_dict['payment']['attributes'][10]['value']['payer']['attributes'][3]['value']['address'][
                'attributes'][1][
                'length'] == 2
        assert \
            parsed_dict['payment']['attributes'][10]['value']['payer']['attributes'][3]['value']['address'][
                'attributes'][2][
                'name'] == 'zip_code'
        assert \
            parsed_dict['payment']['attributes'][10]['value']['payer']['attributes'][3]['value']['address'][
                'attributes'][2][
                'value'] == '99999999'
        assert \
            parsed_dict['payment']['attributes'][10]['value']['payer']['attributes'][3]['value']['address'][
                'attributes'][2][
                'supported']
        assert \
            parsed_dict['payment']['attributes'][10]['value']['payer']['attributes'][3]['value']['address'][
                'attributes'][2][
                'native']
        assert not \
            parsed_dict['payment']['attributes'][10]['value']['payer']['attributes'][3]['value']['address'][
                'attributes'][2][
                'many']
        assert \
            parsed_dict['payment']['attributes'][10]['value']['payer']['attributes'][3]['value']['address'][
                'attributes'][2][
                'data_type'] == 'str'
        assert \
            parsed_dict['payment']['attributes'][10]['value']['payer']['attributes'][3]['value']['address'][
                'attributes'][2][
                'length'] == 8
        assert parsed_dict['payment']['attributes'][10]['value']['payer']['attributes'][3]['supported']
        assert not parsed_dict['payment']['attributes'][10]['value']['payer']['attributes'][3]['native']
        assert not parsed_dict['payment']['attributes'][10]['value']['payer']['attributes'][3]['many']
        assert parsed_dict['payment']['attributes'][10]['value']['payer']['attributes'][3]['data_type'] == 'Address'
        assert parsed_dict['payment']['attributes'][10]['supported']
        assert not parsed_dict['payment']['attributes'][10]['native']
        assert not parsed_dict['payment']['attributes'][10]['many']
        assert parsed_dict['payment']['attributes'][10]['data_type'] == 'Payer'
        assert parsed_dict['payment']['attributes'][11]['name'] == 'order_id'
        assert parsed_dict['payment']['attributes'][11]['value'] == '4m1OdghPUQtg'
        assert parsed_dict['payment']['attributes'][11]['supported']
        assert parsed_dict['payment']['attributes'][11]['native']
        assert not parsed_dict['payment']['attributes'][11]['many']
        assert parsed_dict['payment']['attributes'][11]['data_type'] == 'str'
        assert parsed_dict['payment']['attributes'][11]['length'] == 12
        assert parsed_dict['payment']['attributes'][12]['name'] == 'notification_url'
        assert parsed_dict['payment']['attributes'][12]['value'] == 'http://www.merchant.com/notifications'
        assert parsed_dict['payment']['attributes'][12]['supported']
        assert parsed_dict['payment']['attributes'][12]['native']
        assert not parsed_dict['payment']['attributes'][12]['many']
        assert parsed_dict['payment']['attributes'][12]['data_type'] == 'str'
        assert parsed_dict['payment']['attributes'][12]['length'] == 37
        assert parsed_dict['payment']['attributes'][13]['name'] == 'created_date'
        assert parsed_dict['payment']['attributes'][13]['value'] == '2019-06-26T15:17:31.000+0000'
        assert parsed_dict['payment']['attributes'][13]['supported']
        assert parsed_dict['payment']['attributes'][13]['native']
        assert not parsed_dict['payment']['attributes'][13]['many']
        assert parsed_dict['payment']['attributes'][13]['data_type'] == 'str'
        assert parsed_dict['payment']['attributes'][13]['length'] == 28
        assert parsed_dict['payment']['attributes'][14]['name'] == 'user'
        assert parsed_dict['payment']['attributes'][14]['value']['user']['name'] == 'User'
        assert parsed_dict['payment']['attributes'][14]['value']['user']['level'] == 1
        assert parsed_dict['payment']['attributes'][14]['value']['user']['attributes'][0]['name'] == 'id'
        assert parsed_dict['payment']['attributes'][14]['value']['user']['attributes'][0]['value'] == 1
        assert parsed_dict['payment']['attributes'][14]['value']['user']['attributes'][0]['supported']
        assert parsed_dict['payment']['attributes'][14]['value']['user']['attributes'][0]['native']
        assert not parsed_dict['payment']['attributes'][14]['value']['user']['attributes'][0]['many']
        assert parsed_dict['payment']['attributes'][14]['value']['user']['attributes'][0]['data_type'] == 'int'
        assert parsed_dict['payment']['attributes'][14]['value']['user']['attributes'][1]['name'] == 'username'
        assert parsed_dict['payment']['attributes'][14]['value']['user']['attributes'][1]['value'] == 'bwayne'
        assert parsed_dict['payment']['attributes'][14]['value']['user']['attributes'][1]['supported']
        assert parsed_dict['payment']['attributes'][14]['value']['user']['attributes'][1]['native']
        assert not parsed_dict['payment']['attributes'][14]['value']['user']['attributes'][1]['many']
        assert parsed_dict['payment']['attributes'][14]['value']['user']['attributes'][1]['data_type'] == 'str'
        assert parsed_dict['payment']['attributes'][14]['value']['user']['attributes'][1]['length'] == 6
        assert parsed_dict['payment']['attributes'][14]['value']['user']['attributes'][2]['name'] == 'created'
        assert parsed_dict['payment']['attributes'][14]['value']['user']['attributes'][2]['value'] == datetime(2022, 10,
                                                                                                               21, 11,
                                                                                                               56, 25,
                                                                                                               23)
        assert parsed_dict['payment']['attributes'][14]['value']['user']['attributes'][2]['supported']
        assert parsed_dict['payment']['attributes'][14]['value']['user']['attributes'][2]['native']
        assert not parsed_dict['payment']['attributes'][14]['value']['user']['attributes'][2]['many']
        assert parsed_dict['payment']['attributes'][14]['value']['user']['attributes'][2]['data_type'] == 'datetime'
        assert parsed_dict['payment']['attributes'][14]['supported']
        assert not parsed_dict['payment']['attributes'][14]['native']
        assert not parsed_dict['payment']['attributes'][14]['many']
        assert parsed_dict['payment']['attributes'][14]['data_type'] == 'User'


def test_simple_parsing(output_folder):
    data = {
        "id": "D-4-be8eda8c-5fe7-49dd-8058-4ddaac00611b",
        "amount": 72.00,
        "status": "PAID",
        "status_detail": "The payment was paid.",
        "status_code": "200",
        "currency": "USD",
        "country": "AR",
        "payment_method_id": "RP",
        "payment_method_type": "TICKET",
        "payment_method_flow": "REDIRECT",
        "payer": {
            "name": "Nino Deicas",
            "user_reference": "US-jmh3gb4kj5h34",
            "email": "buyer@gmail.com",
            "address": {
                "street": "123th street",
                "state": "FL",
                "zip_code": "99999999"
            }
        },
        "order_id": "4m1OdghPUQtg",
        "notification_url": "http://www.merchant.com/notifications",
        "created_date": "2019-06-26T15:17:31.000+0000",
        "user": {
            "id": 1,
            "username": "bwayne",
            "created": datetime.now()
        }
    }
    # 1. Parse raw dictionary
    parsed_dict = parse_dict(data)
    quick_write(parsed_dict, 'parsed.json')
    # 2. Transform dictionary to a list of models
    model_list = transform_dict_to_model_list(parsed_dict)
    model_list = parse_for_patterns(model_list, PATTERN_FUNCTIONS)
    model_list = build_serializer_data(model_list)

    quick_write(model_list, 'model_list.json')
    template_model_list = build_serializer_template_data(model_list)
    quick_write(template_model_list, 'template_model_list.json')

    writer = ReportWriter()
    output_file = output_folder / 'serializers.py'
    writer.write('drf_serializers.py.j2', output_file, template_data=template_model_list)
    assert output_file.exists()


def test_simple_parsing_camel_case(output_folder, camel_case_dict):
    prefix = 'camel_case'
    parsed_dict = parse_dict(camel_case_dict)
    quick_write(parsed_dict, f'{prefix}_parsed.json')

    model_list = transform_dict_to_model_list(parsed_dict)
    model_list = parse_for_patterns(model_list, PATTERN_FUNCTIONS)
    kmodel_list = build_serializer_data(model_list)

    quick_write(model_list, f'{prefix}_model_list.json')

    template_model_list = build_serializer_template_data(model_list)
    quick_write(template_model_list, f'{prefix}_serializer_template_data.json')

    writer = ReportWriter()
    output_file = output_folder / f'{prefix}_serializers_camel_case.py'
    writer.write('drf_serializers.py.j2', output_file, template_data=template_model_list)
    assert output_file.exists()


def test_attributes_with_list(output_folder, fixtures_folder):
    filename = fixtures_folder / 'api_bulk.json'
    with open(filename, 'r') as json_file:
        raw_data_dict = json.load(json_file)
    prefix = '_with_list'

    parsed_dict = parse_dict(raw_data_dict)
    quick_write(parsed_dict, f'{prefix}_parsed.json')

    model_list = transform_dict_to_model_list(parsed_dict)
    model_list = parse_for_patterns(model_list, PATTERN_FUNCTIONS)
    model_list = build_serializer_data(model_list)

    quick_write(model_list, f'{prefix}_model_list.json')

    template_model_list = build_serializer_template_data(model_list)
    quick_write(template_model_list, f'{prefix}_serializer_template_data.json')

    writer = ReportWriter()
    output_file = output_folder / f'{prefix}_serializers_camel_case.py'
    writer.write('drf_serializers.py.j2', output_file, template_data=template_model_list)
    assert output_file.exists()


def test_parsefile_for_ast_classes(fixtures_folder, output_folder):
    module_file = 'pydantic_models.py'
    filename = fixtures_folder / module_file

    ast_module = parse_file_for_ast_classes(filename)
    quick_write(ast_module, f'{module_file}.json')


class TestParseVarName:

    def test_parse_camel_case(self):
        varname = 'mySpecialVariable'
        snake_case_varname, was_changed = parse_var_name(varname)
        assert snake_case_varname == 'my_special_variable'
        assert was_changed

    def test_parse_pascal_case(self):
        varname = 'PaymentSerializer'
        snake_case_varname, was_changed = parse_var_name(varname)
        assert snake_case_varname == 'payment_serializer'
        assert was_changed

    def test_parse_snake_case(self):
        varname = 'flow_id'
        snake_case_varname, was_changed = parse_var_name(varname)
        assert snake_case_varname == 'flow_id'
        assert not was_changed

    @pytest.mark.parametrize('varname, expected',
                             [('IMEIValue', 'imei_value'),
                              ('DGINumberId', 'dgi_number_id')])
    def test_parse_with_acronym(self, varname, expected):
        snake_case_varname, was_changed = parse_var_name(varname)
        assert snake_case_varname == expected
        assert was_changed


def test_to_camel_case():
    data = {
        "amount": 100,
        "payer": {
            "fullName": "Nichole Stevens",
            "email": "ljones@example.org",
            "document": "07323861808"
        },
        "merchant_id": "something",
        "phone_number": "56985845"
    }
    dict_camelized = humps.camelize(data)
    # quick_write(dict_camelized, 'camelized_dict.json')
    assert dict_camelized['merchantId'] == data['merchant_id']
    assert dict_camelized['phoneNumber'] == data['phone_number']
    assert dict_camelized['payer']['fullName'] == data['payer']['fullName']
def test_parse_file_for_enum(fixtures_folder):
    csv_file = fixtures_folder / 'enum_str.csv'
    results = parse_file_for_enum(csv_file)



from django_scaffolding_tools.django.builders import build_model_serializer_template_data
from django_scaffolding_tools.django.parsers import parse_for_django_classes
from django_scaffolding_tools.parsers import parse_file_for_ast_classes
from django_scaffolding_tools.utils.core import quick_write


def test_build_model_serializer_template_data_camelize(output_folder, fixtures_folder):
    model_filename = 'simple_models2.py'
    filename = fixtures_folder / model_filename
    # 1 Convert model.py to an ast json file.
    ast_dict = parse_file_for_ast_classes(filename)
    # 2 Parse AST json dictionary for Django Model data
    model_data = parse_for_django_classes(ast_dict)
    # 3 Build serializer data form Django model data
    serializer_data = build_model_serializer_template_data(model_data, add_source_camel_case=True)

    quick_write(ast_dict, f'ast_{model_filename}.json', output_subfolder='django')
    quick_write(model_data, f'model_data_{model_filename}.json', output_subfolder='django')
    quick_write(serializer_data, f'serializer_data_{model_filename}.json', output_subfolder='django')

    assert len(serializer_data) == 2

    model1 = serializer_data[0]
    assert model1['model'] == 'Clinic'
    assert model1['fields'][0]['name'] == 'name'
    assert model1['fields'][0]['serializer'] == 'serializers.CharField()'

    model2 = serializer_data[1]
    assert model2['model'] == 'Patient'
    assert model2['fields'][0]['name'] == 'first_name'
    assert model2['fields'][0]['source'] == 'firstName'
    assert model2['fields'][0]['serializer'] == 'serializers.CharField(source=\'firstName\')'
    assert model2['fields'][1]['name'] == 'last_name'
    assert model2['fields'][1]['source'] == 'lastName'
    assert model2['fields'][1]['serializer'] == 'serializers.CharField(source=\'lastName\')'
    assert model2['fields'][2]['name'] == 'owning_clinic'
    assert model2['fields'][2]['source'] == 'owningClinic'
    assert model2['fields'][2]['serializer'] == 'ClinicSerializer(source=\'owningClinic\', read_only=True)'


def test_build_model_serializer_template_data(output_folder, fixtures_folder):
    model_filename = 'simple_models2.py'
    filename = fixtures_folder / model_filename
    # 1 Convert model.py to an ast json file.
    ast_dict = parse_file_for_ast_classes(filename)
    # 2 Parse AST json dictionary for Django Model data
    model_data = parse_for_django_classes(ast_dict)
    # 3 Build serializer data form Django model data
    serializer_data = build_model_serializer_template_data(model_data, add_source_camel_case=False)

    quick_write(ast_dict, f'ast_{model_filename}.json', output_subfolder='django')
    quick_write(model_data, f'model_data_{model_filename}.json', output_subfolder='django')
    quick_write(serializer_data, f'serializer_data_{model_filename}.json', output_subfolder='django')

    assert len(serializer_data) == 2

    model1 = serializer_data[0]
    assert model1['model'] == 'Clinic'
    assert model1['fields'][0]['name'] == 'name'
    assert model1['fields'][0]['serializer'] == 'serializers.CharField()'

    model2 = serializer_data[1]
    assert model2['model'] == 'Patient'
    assert model2['fields'][0]['name'] == 'first_name'
    assert model2['fields'][0]['serializer'] == 'serializers.CharField()'
    assert model2['fields'][1]['name'] == 'last_name'
    assert model2['fields'][1]['serializer'] == 'serializers.CharField()'
    assert model2['fields'][2]['name'] == 'owning_clinic'
    assert model2['fields'][2]['serializer'] == 'ClinicSerializer()'



import subprocess
from typing import List, Tuple


def run_commands(commands: List[str], encoding: str = 'utf-8') -> Tuple[List[str], List[str]]:
    """
    :param commands: <list> The command and paraemters to run
    :param encoding: <str> Encoding for the shell
    :return: <tuple> Containing 2 lists. First one with results and the Second one with errors if any.
    """
    result = subprocess.run(commands,
                            stdout=subprocess.PIPE,
                            stderr=subprocess.PIPE,
                            stdin=subprocess.PIPE)
    result_lines = result.stdout.decode(encoding).split('\n')[:-1]
    error_lines = result.stderr.decode(encoding).split('\n')[:-1]
    return result_lines, error_lines


def run_command_with_grep(commands: List[str], regexp: str) -> List[str]:
    ps = subprocess.run(commands, check=True, capture_output=True)
    # print(ps.stderr.decode('utf-8').strip())
    containers = subprocess.run(['grep', '-E', f'{regexp}'],
                                input=ps.stdout, capture_output=True)
    results = containers.stdout.decode('utf-8').strip().split('\n')
    return results



import logging
from pathlib import Path

from django_scaffolding_tools._experimental.reqs_utilities.parsers import parse_requirement_file
from django_scaffolding_tools._experimental.reqs_utilities.db import RequirementDatabase

logger = logging.getLogger(__name__)


class Updater:

    def __init__(self, database: RequirementDatabase):
        self.database = database

    def update_requirements(self, requirement_file: Path):
        reqs = parse_requirement_file(requirement_file)
        lines = list()
        logger.debug(f'Ready to parse {len(reqs)} requirement.')
        for req in reqs:
            if req['parsed'] is None:
                lines.append(req['raw'])
            else:
                recommended = self.database.get(req['parsed']['lib_name'])
                if recommended is None:
                    lines.append(req['raw'])
                else:
                    line = recommended.to_req_line()
                    lines.append(f'{line}\n')

        with open(requirement_file, 'w') as r_file:
            string_lines = ''.join(lines)
            r_file.write(string_lines)


if __name__ == '__main__':
    home = Path().home()
    db_file = Path(__file__).parent / 'req_db.json'
    db = RequirementDatabase(db_file)
    output_folder = Path(__file__).parent.parent.parent.parent / 'output'
    # project = 'adelantos-cupos'
    # project = 'ec-d-local-payment-collector'
    # project = 'payment-queue-processor'
    # project = 'credibanco_integration'
    # project = 'movil-reseller-payments'
    # project = 'sms-integration'
    #project = 'payment_router'
    # project = 'multipagos-integrator'
    # project ='pj_django_payments/tests/example'
    project = 'payment-collector'
    command = 'CHANGE'
    if command == 'CHANGE':
        updater = Updater(db)
        files = ['local.txt', 'base.txt', 'production.txt', 'staging.txt']
        for file in files:
            f = home / f'adelantos/{project}/requirements/{file}'
            if f.exists():
                updater.update_requirements(f)



import uuid

from typing import Any, Dict

from django.db import models
from django.utils.translation import ugettext_lazy as _
# Create your models here.
from django_fsm import FSMField, transition
from model_utils.models import TimeStampedModel
from pj_d_local_sdk.models import PaymentResponse
from pj_d_local_sdk.pay_ins.payments import PaymentManager

from .d_local import send_payment_to_d_local
from .enums import PaymentState

STATE_CHANGES = 'state_changes'


def get_response_default_dict() -> Dict[str, Any]:
    return {'request': '', 'callback': ''}


def get_metadata_default_dict() -> Dict[str, Any]:
    """Default dictionary to keep track of changes in state. Add a dictionary containing the date the state
    changed. For example:
    metadata['state_changes'][PaymentState.SENT_TO_PROVIDER.value].append({'date':'2022-08-08T15:45:00+00:00'})"""
    metadata = dict()
    metadata[STATE_CHANGES] = dict()
    metadata[STATE_CHANGES][PaymentState.SENT_TO_PROVIDER.value] = []
    metadata[STATE_CHANGES][PaymentState.PROVIDER_RESPONDED.value] = []
    metadata[STATE_CHANGES][PaymentState.RESPONDED_TO_CALLER.value] = []
    metadata[STATE_CHANGES][PaymentState.NOTIFICATION_RECEIVED.value] = []
    metadata[STATE_CHANGES][PaymentState.NOTIFICATION_SENT_TO_CALLER.value] = []
    metadata[STATE_CHANGES][PaymentState.CLOSED.value] = []

    return metadata


class Payer(TimeStampedModel):
    """D-Local representation of a Customer. The country attribute was added to avoid future collitions with
    repeated country. This occurred in between Colombia and Peru national ids."""
    name = models.CharField(_('Name'), max_length=100)
    email = models.CharField(_('Email'), max_length=100)
    document = models.CharField(_('Document'), max_length=100,
                                help_text=_('User’s personal identification number. '
                                            'To see the document code list per country, '
                                            'go to the Country Reference page.'))
    country = models.CharField(_('Country'), max_length=2,
                               help_text=_('This will contain the two letter country code'
                                           ' of the country the device was sold in.'))

    def __str__(self):
        return f'{self.name} ({self.document})'

    class Meta:
        unique_together = ('document', 'country')


class Payment(TimeStampedModel):
    """Represents a payment sent from M3 and resolved or rejected by D-Local."""
    payer = models.ForeignKey(Payer, verbose_name=_('Payer'), on_delete=models.PROTECT,
                              related_name='payments')
    credit_line_id = models.CharField(_('Credit line id'), max_length=64,
                                      help_text=_('Unique identifier for a credit line or a finance order.'))
    finance_engine_version = models.CharField(_('Finance engine version'), max_length=5,
                                              help_text=_('Finance engine version FEv1 or FEv2'))
    user_id = models.CharField(_('User id'), max_length=50,
                               help_text=_('This will contain the foreign key to the Users table in '
                                           'M2 or the Persons table in M3'))
    device_id = models.CharField(_('Device id'), max_length=50,
                                 help_text=_('This will contain the foreign key to the Devices table'))
    merchant_id = models.CharField(_('Merchant id'), max_length=64, null=True, blank=True,
                                   help_text=_('Reference to the Merchants table - ONLY '
                                               'FEv2 needs to publish this information'))
    clerk_id = models.CharField(_('Clerk id'), max_length=50, null=True, blank=True,
                                help_text=_('This will contain the foreign key to the clerk who sold the device'))
    finance_provider = models.CharField(_('Finance provider'), max_length=50,
                                        help_text=_('Constants representing the finance provider.'))
    payment_provider = models.CharField(_('Payment provider'), max_length=16)
    amount = models.DecimalField(_('Amount'), max_digits=12, decimal_places=2)
    currency = models.CharField(_('Currency'), max_length=3,
                                help_text=_("The three letter currency code abbreviation."))
    country = models.CharField(_('Country'), max_length=2,
                               help_text=_('This will contain the two letter country code'
                                           ' of the country the device was sold in.'))
    payment_method_id = models.CharField(_('Payment method id'), max_length=3,
                                         help_text=_('Payment method defined by D-Local.'))
    payment_method_flow = models.CharField(_('Payment method flow'), max_length=15,
                                           help_text=_('D-Local payment flow usually DIRECT or REDIRECT.'))
    order_id = models.UUIDField(primary_key=False, unique=True, default=uuid.uuid4, editable=False)
    notification_url = models.URLField(_('Notification url'))
    payment_status = models.CharField(_('Payment status'), max_length=15, null=True, blank=True,
                                      help_text=_('The values should match pj_d_local_sdk.enums.PaymentStatus values.'))

    response = models.JSONField(_('Response'), default=get_response_default_dict,
                                help_text=_("Response from D-Local API. Will contain 2 keys for "
                                            "request response and callback response"))
    state = FSMField(_('State'), protected=True, default=PaymentState.CREATED)
    metadata = models.JSONField(_('Metadata'), default=get_metadata_default_dict,
                                help_text=_('Metadata for the payment. It includes the dates with'
                                            ' the changes of state.'))

    @transition(field=state, source=PaymentState.CREATED, target=PaymentState.SENT_TO_PROVIDER)
    def send_to_provider(self, payment_manager_instance: PaymentManager) -> PaymentResponse:
        """This method will call the D-Local endpoint and return a pj_d_local_sdk.models.PaymentResponse object"""
        payment_response = send_payment_to_d_local(self, payment_manager_instance)
        return payment_response

    @transition(field=state, source=PaymentState.SENT_TO_PROVIDER, target=PaymentState.RESPONDED_TO_CALLER)
    def responded_m3(self):
        pass

    def __str__(self):
        return f'Order id: {self.order_id}'

    class Meta:
        ordering = ('-created',)



import json
import re
from pathlib import Path
from typing import Dict, Any, List

from django_scaffolding_tools._experimental.reqs_utilities.db import RequirementDatabase
from django_scaffolding_tools._experimental.reqs_utilities.models import convert_version_to_tuples


# def parse_for_permitted_libs(req_file: Path):
#     regexp = re.compile(r'(?P<lib_name>[\w_\-]+)==(?P<version>[\w\.\-]+)\s*#?(?P<comment>.*)')
#     with open(req_file, 'r') as r_file:
#         lines = r_file.readlines()
#     parsed_requirements = dict()
#     for i, line in enumerate(lines, 1):
#         match = regexp.match(line)
#         if match:
#             lib_name = match.group('lib_name')
#             parsed_requirements[lib_name] = match.group('version')
#     return parsed_requirements


def parse_requirement_file(req_file: Path) -> List[Dict[str, Any]]:
    regexp = re.compile(r'(?P<lib_name>[\w_\-]+)==(?P<version>[\w\.\-]+)\s*#?(?P<comment>.*)')
    with open(req_file, 'r') as r_file:
        lines = r_file.readlines()
    var_names = ['lib_name', 'version', 'comment']
    parsed_requirements = list()
    for i, line in enumerate(lines, 1):
        match = regexp.match(line)
        req = {'raw': line, 'line_number': i, 'parsed': None}
        if match:
            req['parsed'] = dict()
            for var_name in var_names:
                req['parsed'][var_name] = match.group(var_name)
        parsed_requirements.append(req)
    return parsed_requirements

def interactive_parse_requirements(req_file:Path, db: RequirementDatabase):
    requirements = parse_requirement_file(req_file)
    for requirement in requirements:
        if requirement.get('parsed'):
            name = requirement['parsed']['lib_name']
            version = requirement['parsed']['version']
            db_requirement = db.get(name)
            if db_requirement is not None:
                version_info = convert_version_to_tuples(version)
                if version_info <= db_requirement.approved_version_info:
                    print(f"{name} - {version}  {db_requirement.latest_version} {db_requirement.approved_version}")
            else:
                add_to_db = input(f'Add {name} version {version} to db?')
                if add_to_db.upper() == 'Y':
                    db.add(name, None)

def save_requirements_to_json(filename: Path, folder: Path):
    out_file = folder / 'reqs.json'
    reqs = parse_requirement_file(filename)

    with open(out_file, 'w') as d_file:
        json.dump(reqs, d_file)


def main2(requirements_folder: Path, folder: Path):
    out_file = folder / 'permitted.json'
    req_files = requirements_folder.glob('**/*.txt')
    global_req = dict()
    for req_file in req_files:
        reqs = parse_for_permitted_libs(req_file)
        global_req.update(reqs)
    with open(out_file, 'w') as d_file:
        json.dump(global_req, d_file)


if __name__ == '__main__':
    home = Path().home()
    # db_file = home / 'PycharmProjects/django_scaffolding_tools/tests/fixtures/_experimental/req_db.json'
    db_file = Path(__file__).parent / 'req_db.json'
    if not db_file.exists():
        raise Exception('File not found')
    db = RequirementDatabase(db_file)


    project = 'adelantos-cupos'
    project = 'ec-d-local-payment-collector'
    # project = 'payment-queue-processor'
    # project = 'credibanco_integration'
    # project = 'movil-reseller-payments'
    # project = 'sms-integration'
    # project = 'payment_router'
    command = 'UPDATE'
    files = ['local.txt', 'base.txt', 'production.txt']
    for file in files:
        f = home / f'adelantos/{project}/requirements/{file}'
        interactive_parse_requirements(f, db)



import json
import os
import sys
from pathlib import Path

import click

from django_scaffolding_tools.django.utils import DjangoAppManager
from django_scaffolding_tools.django.writers import write_model_serializer_from_models_file


@click.command()
@click.argument('command')
@click.option('--folder', default=os.getcwd())
@click.option('--model-file', default='models.py')
@click.option('--output-folder', default=None)
@click.option('--output-file', default=None)
@click.option('--camel-case', is_flag=True, default=False)
def main(command, folder, model_file, output_folder, output_file, camel_case):
    """Console script for django_scaffolding_tools."""
    click.echo(f"Current directory {os.getcwd()}")
    if command.lower() == 'model_2_serializer':
        click.echo(f'folder: {folder}')
        model_full_path = Path(folder) / model_file
        click.echo(f'model_file: {model_file} exists {model_full_path.exists()}')
        output_folder = Path(output_folder)
        click.echo(f'output_folder: {output_folder} exists: {output_folder}')
        if output_file is None:
            output_file = output_folder / f'{command}.py'
        click.echo(f'output_file: {output_file}')
        write_model_serializer_from_models_file(model_full_path, output_file, camel_case=camel_case)



@click.command()
@click.option('--settings', default='config.settings.local')
@click.option('--output-folder', default=Path(os.getcwd()) / 'output', type=click.Path(exists=True))
@click.option('--app')
def models_to_json(app, output_folder, settings):
    def quick_serialize(value):
        return f'{value}'

    os.environ.setdefault("DJANGO_SETTINGS_MODULE",settings)
    import django
    django.setup()
    manager = DjangoAppManager()
    # print(list(manager.get_installed_apps()))
    app_data = manager.get_app_data(app)

    print(app_data)
    filename = output_folder / f'{app}.json'
    with open(filename, 'w') as json_file:
        json.dump(app_data, json_file, indent=4, default=quick_serialize)
    click.echo(f'Save file {filename}')


if __name__ == "__main__":
    sys.exit(main())  # pragma: no cover



from django.db import models
from django.utils.translation import ugettext_lazy as _
from django.config import settings

# Create your models here.


class Clinic(models.Model):
    name = models.CharField(_('Name'), max_length=100)
    created_by = models.ForeignKey(settings.AUTH_USER_MODEL, null=True, blank=True, verbose_name=_('Created by'),
                                   help_text=_('User who created the record'), on_delete=models.PROTECT,
                                   related_name='contracts_created')


class Patient(models.Model):
    first_name = models.CharField(_('First name'), max_length=100, help_text="Patient's fist name")
    last_name = models.CharField(_('Last name'), max_length=100)
    owning_clinic = models.ForeignKey(Clinic, verbose_name=_('Clinic'), related_name='companies',
                                      on_delete=models.PROTECT)



import os
from enum import Enum
from pathlib import Path
from typing import Dict, Any

import toml


class ConfigurationManager:

    def __init__(self, config_folder: Path = None):
        if config_folder is None:
            self.config_folder = Path().home() / '.py_cover_letters'
            self.config_folder.mkdir(exist_ok=True)
        self.config_file = self.config_folder / 'configuration.toml'
        self.username = os.getlogin()

    def get_sample_config(self):
        data = {'cover_letters': {'template_folder': str(self.config_folder / 'templates'),
                                  'default_template': 'Cover Letter Template.docx',
                                  'default_output_folder': str(Path(os.getcwd()) / 'output')},
                'gmail': {'email': f'{self.username}@gmail.com', 'token': 'SECRET'},
                'database': {'folder': str(Path(os.getcwd()) / 'data'), 'file': 'cover_letters.xlsx'}}
        return data

    def write_configuration(self, config_data: Dict[str, Any], over_write=False):
        if self.config_file.exists() and not over_write:
            raise Exception('Cannot overwrite config file.')
        with open(self.config_file, 'w') as f:
            toml.dump(config_data, f)

    def get_configuration(self):
        with open(self.config_file, 'r') as f:
            configuration = toml.load(f)
        return configuration

    @classmethod
    def get_current(cls):
        config = cls()
        return config.get_configuration()


def main(install: True):
    config_manager = ConfigurationManager()
    configuration = config_manager.get_sample_config()
    email = configuration['gmail']['email']
    new_email = input(f'Sender email <{email}>: ')
    if new_email != '':
        configuration['gmail']['email'] = new_email

    gmail_token = input('GMail token: ')
    configuration['gmail']['token'] = gmail_token
    print(configuration)
    config_manager.write_configuration(configuration, over_write=True)


if __name__ == '__main__':
    main(True)



from enum import Enum


class ProjectPostgresRegExp(str, Enum):
    PCA = r"payment[_\-]collector[a-z_\-]*_postgres"
    ECDL = r"d[_\-]local[a-z_\-]*_postgres"
    LMS = r"[_\-]lite[a-z_\-]*_postgres"
    CUP = r"[_\-]cupos[a-z_\-]*_postgres"
    MIN = r"[_\-]?mobile[a-z_\-]*_postgres"
    SLACK = r"[_\-]?slack_[a-z_\-]*_postgres"
    PQP = r"[_\-]?queue_[a-z_\-]*_postgres"
    PJ_PAYMENT = r"[_\-]?pj_django_payments[a-z_\-]*_postgres"
    MPI = r"multipagos[a-z_\-]*_postgres"
    PRO = r"payment[_\-]router[a-z_\-]*_postgres"


class TerminalColor(str, Enum):
    HEADER = '\033[95m'
    OK_BLUE = '\033[94m'
    OK_CYAN = '\033[96m'
    OK_GREEN = '\033[92m'
    WARNING = '\033[93m'
    ERROR = '\033[91m'
    END_COLOR = '\033[0m'
    BOLD = '\033[1m'
    UNDERLINE = '\033[4m'



from pathlib import Path

from django_scaffolding_tools.django.writers import write_model_serializer_from_models_file, \
    write_model_factories_from_models_file


def test_write_model_serializer_from_models_file(fixtures_folder, output_folder):
    models_file = fixtures_folder / 'finance_models.py'
    serializer_file = output_folder / 'django' / 'serializers.py'
    write_model_serializer_from_models_file(models_file, serializer_file, camel_case=False)


def test_write_model_serializer_from_models_help(fixtures_folder, output_folder):
    models_file = fixtures_folder / 'models_with_helptext.py'
    models_file = output_folder / 'models.py'
    models_file = Path('/home/luiscberrocal/adelantos/bcp-integration/bcp_integration/api/models.py')
    serializer_file = output_folder / 'django' / 'serializers_2.py'
    write_model_serializer_from_models_file(models_file, serializer_file, write_intermediate=True, camel_case=False)

def test_write_model_factories_from_models_file(output_folder):
    models_file = Path('/home/luiscberrocal/PycharmProjects/alpha_clinic/alpha_clinic/patients/models.py')
    output_file = output_folder / 'django' / 'factories.py'
    write_model_factories_from_models_file(models_file, output_file, write_intermediate=True)



import json
import os
import pickle
from pathlib import Path

from django_scaffolding_tools._experimental.backup_envs.gdrive import get_g_drive_credentials


def main():
    drive = get_google_drive()

    # Auto-iterate through all files in the root folder.
    folder = 'root'
    file_list = drive.ListFile({'q': f"'{folder}' in parents and trashed=false"}).GetList()

    for file1 in file_list:
        print('title: %s, id: %s kind: %s' % (file1['title'], file1['id'], file1['kind']))


def get_google_drive():
    from pydrive2.auth import GoogleAuth
    from pydrive2.drive import GoogleDrive

    secrets_folder = Path(__file__).parent.parent.parent.parent / '.envs' / 'google_drive'
    secrets_file = secrets_folder / 'client_secrets.json'
    token_file = secrets_folder / 'token2.json'

    GoogleAuth.DEFAULT_SETTINGS['client_config_file'] = str(secrets_file)
    gauth = GoogleAuth()
    if token_file.exists():
        gauth.LoadCredentialsFile(token_file)
    else:
        gauth.LocalWebserverAuth()
        gauth.SaveCredentialsFile(token_file)

    drive = GoogleDrive(gauth)
    return drive


def upload(folder: Path, google_drive_folder: str):
    drive = get_google_drive()
    zip_files = folder.glob('**/*.zip')
    for i, zip_file in enumerate(zip_files, 1):
        print(f'{i} {zip_file.name}')
        gfile = drive.CreateFile({'parents': [{'id': google_drive_folder}], 'title': zip_file.name})
        # Read file and set it as the content of this instance.
        gfile.SetContentFile(str(zip_file))
        gfile.Upload()  # Upload the file.


def list_directories(folder: Path, top: int = 3):
    env_folders = list()
    for root, dirs, files in os.walk(folder):
        dirs.reverse()
        filtered = dirs[:top]
        for directory in filtered:
            env_folders.append(directory)
            # print(directory)
    return env_folders


def list_dated_directories(folder: Path, top: int = 3):
    directories = list_directories(folder, top)


if __name__ == '__main__':
    # main()

    g_drive_folder = Path(__file__).parent.parent.parent.parent / '.envs' / 'google_drive'
    folder_ids_file = g_drive_folder / 'google_drive_folders_id.json'
    with open(folder_ids_file, 'r') as json_file:
        folder_ids = json.load(json_file)
    folder_id = folder_ids['Envs']

    envs_folder = Path('/home/luiscberrocal/Documents/adelantos_envs')
    backup_folders = list_directories(envs_folder, top=5)
    last_folder = list_directories(envs_folder, top=3)[0]

    z_folder = envs_folder / last_folder
    prompt = input(f'Upload {last_folder} to gdrive [y/N]? ')
    if prompt.lower() == 'y':
        upload(z_folder, folder_id)



from datetime import datetime
from pathlib import Path

import humps

from django_scaffolding_tools._experimental.cover_letters.config import ConfigurationManager
from django_scaffolding_tools._experimental.cover_letters.generators import write_docx_cover_letter, convert_docx_to_pdf


def test_write_cover_letter(fixtures_folder, output_folder):
    template = fixtures_folder / '_experimental' / 'Cover Letter Template.docx'
    today = datetime.today()
    context = {'date': today.strftime('%b %M %Y'), 'position_name': 'Jedi Knight',
               'company_name': 'Jedi Order Council'}

    docx_filename = f'{today.strftime("%Y%m%d")}_cover_{context["company_name"]}_{context["position_name"]}.docx'
    cover_letter = output_folder / docx_filename
    cover_letter.unlink(missing_ok=True)

    write_docx_cover_letter(template, context, cover_letter)
    assert cover_letter.exists()


def test_convert_docx_to_pdf(fixtures_folder, output_folder):
    config = ConfigurationManager()
    configuration = config.get_configuration()
    template = Path(configuration['cover_letters']['template_folder']) / configuration['cover_letters']['default_template']
    # template = fixtures_folder / '_experimental' / 'Cover Letter Template.docx'
    today = datetime.today()
    context = {'date': today.strftime('%B %-d, %Y'), 'position_name': 'Jedi Knight',
               'company_name': 'Jedi Order Council'}
    naming_context = context  # humps.camelize(context)
    docx_filename = f'{today.strftime("%Y%m%d")}_cover_{naming_context["company_name"]}' \
                    f'_{naming_context["position_name"]}.docx'
    cover_letter = output_folder / docx_filename
    cover_letter.unlink(missing_ok=True)

    write_docx_cover_letter(template, context, cover_letter)
    assert cover_letter.exists()

    pdf = convert_docx_to_pdf(cover_letter, output_folder)
